################################################################################
### Name:     AllFullGDM.R                                                   ###
### aim:                                                                     ###
### Run all the GDM models with the full data (all sites pairs).             ###
###                                                                          ###
### Input:                                                                   ###
### Data      - main data frame with all the data required for the gdm.      ###
### Details   - Data frame, Output of the MakePairsFull function.            ###
### Depen     - the column number to use as the depndent variable.           ###
### Vars      - numeric vector with the column numbers with the basic 6      ###
###             variables of GDM:  'distance', 'weights', 's1.xCoord',       ###         
###             's1.yCoord', 's2.xCoord', and 's2.yCoord'.                   ###
### Method    - Method by which the dependent variable is introduced into    ###
###             GDM:                                                         ###
###             "AsIS"       - the raw values are entered.                   ###
###             "Comp1"      - the complimentary to 1 is used.               ###
###             "Stan11"     - the range[-1, 1] is standardaised to the      ###
###                            range [0.1] as (value + 1)/2.                 ###
###             "CompStan11" - the range[-1, 1] is standardaised to the      ###
###                            range [0.1] as 1-((value + 1)/2).             ###
### NMDS       - Logical, if TRUE, the nMDS results for the spectral and     ###
###              environmental variables are used instead of the raw         ###
###              variables.                                                  ###
### geo, splines, knots - parameters of the gdm() function.                  ###
### Run       - the number of the Run (same as RunAllRF).                    ###
### FunDir    - Character, the directory with the source functions.          ###
### ResDir    - Character, the directory in which to write the results.      ###
### ToFile    - Logical, if T, some results are written to file.             ###
### verbose   - Logical, should progress be printed on consule.              ###
###                                                                          ###
### Process:                                                                 ###
### 1. Runs the SingleGDM function on:                                       ###
###    1.a. - Spatial  + Temporal + Spectral + Environmental                 ###
###    1.b. - Spatial                                                        ###
###    1.c. - Temporal                                                       ###
###    1.d. - Spectral                                                       ###
###    1.e. - Environmental                                                  ### 
###    1.f. - Spatial  + Temporal                                            ### 
###    1.g. - Spatial  + Spectral                                            ### 
###    1.h. - Spatial  + Environmental                                       ### 
###    1.i. - Temporal + Spectral                                            ### 
###    1.j. - Temporal + Environmental                                       ### 
###    1.k. - Spectral + Environmental                                       ### 
###    1.l. - Spatial  + Temporal + Spectral                                 ### 
###    1.m. - Spatial  + Temporal + Environmental                            ### 
###    1.n. - Spatial  + Spectral + Environmental                            ### 
###    1.o. - Temporal + Spectral + Environmental                            ###
### 2. For each SingleGDM:                                                   ###
###    2.a. - Collect the predcited, ecological distances + backtransoform   ###
###           to kappa.                                                      ###
###    2.b. - Write the ISplines to file.                                    ###
###    2.c. - Collect all the SumGDM values.                                 ###
###                                                                          ###
### Output:                                                                  ###
### A list with 2 objects:                                                   ###
### Data     - Same as input 'Data' with the 3 dditional columns for the     ###
###            predcited distance, predicted acological distance and         ###
###            predcited kappa for each of the 15 GDM models. Columnnmae     ###
###            include infor on variables used.                              ###
### SumGDM   - Info on the GDM models + goodness of fit measures +           ###
###            coefficients + knots for each of the 15 GDM models.           ###
### ModInfo  - Dataframe with the models names and 0,1 table for spatial,    ###
###            temporal, spectral and environmental.                         ###
###                                                                          ###
### Author: Yoni Gavish <gavishyoni@gmail.com>                               ###
### Date:   23 Sep. 2016                                                     ###
################################################################################

AllFullGDM = function(Data,
                      Details,
                      Depen,
                      Vars,
                      Method  = "CompStan11",
                      NMDS    = T, 
                      geo     = F,
                      splines = NULL,
                      knots   = NULL,
                      Run, 
                      FunDir,
                      ResDir,
                      ToFile,
                      verbose = T){
  
  ##############################################################################
  #### required packages                                                     ###
  ##############################################################################
  require(gdm)
  
  ##############################################################################
  #### source functions                                                      ###
  ##############################################################################
  # The Join_Levels function
  
  setwd(FunDir)
  source("SingleGDM.R")
  
  
  
  if(verbose){
    #cat(paste("\n##-----------------------------------------------------##"))
    cat(paste("\n##--- AllFullGDM: --> \n"))
  }
  
  ##############################################################################
  #### Step1 - run the model with all potential variables                    ###
  ##############################################################################
  
  if(NMDS){
    Details2 <- Details[Details$Group %in% c("Spatial", 
                                             "Temporal", 
                                             "SpecNMDS", 
                                             "EnviNMDS"),]
  }
  if(!NMDS){
    Details2 <- Details[Details$Group %in% c("Spatial", 
                                             "Temporal", 
                                             "Spectral", 
                                             "Environmental"),]
  }
  
  ModName <- "AllSites_All"
  
  ModInfo               <- data.frame()
  
  ModInfo[1, "ModName"]       <- ModName
  ModInfo[1, "Spatial"]       <- 1
  ModInfo[1, "Temporal"]      <- 1
  ModInfo[1, "Spectral"]      <- 1
  ModInfo[1, "Environmental"] <- 1
  
  Temp1 <- SingleGDM(Data    = Data,
                     NewData = NULL,
                     Depen   = Depen,
                     Vars    = c(Vars, Details2[, 4], Details2[, 5]),
                     Method  = Method,
                     geo     = geo,
                     splines = splines,
                     knots   = knots,
                     Run     = Run, 
                     ModName = ModName,
                     verbose = verbose)
  # write Isplines to file
  if(ToFile){
    Temp2 <- Temp1[["ISplines"]]
    Name  <- paste("R", Run, "_",ModName, "_Splines.csv", sep="")
    setwd(ResDir)
    write.table(Temp2, Name, sep=",", row.names=FALSE, col.names=TRUE)
    rm(Temp2, Name)
  } # end ToFile if
  #update the Data
  Data <- Temp1[["Data"]]
  #start the SumGDM
  SumGDM <- Temp1[["SumGDM"]]
  ROW <- 2
  rm(Temp1)
  
  ##############################################################################
  #### Step2 - Spatial Only                                                  ###
  ##############################################################################
  
  Details2 <- Details[Details$Group %in% c("Spatial"),]
  ModName <- "AllSites_Spat"
  
  ModInfo[2, "ModName"]       <- ModName
  ModInfo[2, "Spatial"]       <- 1
  ModInfo[2, "Temporal"]      <- 0
  ModInfo[2, "Spectral"]      <- 0
  ModInfo[2, "Environmental"] <- 0
  
  Temp1 <- SingleGDM(Data    = Data,
                     NewData = NULL,
                     Depen   = Depen,
                     Vars    = c(Vars, Details2[, 4], Details2[, 5]),
                     Method  = Method,
                     geo     = geo,
                     splines = splines,
                     knots   = knots,
                     Run     = Run, 
                     ModName = ModName,
                     verbose = verbose)
  # write Isplines to file
  if(ToFile){
    Temp2 <- Temp1[["ISplines"]]
    Name  <- paste("R", Run, "_",ModName, "_Splines.csv", sep="")
    setwd(ResDir)
    write.table(Temp2, Name, sep=",", row.names=FALSE, col.names=TRUE)
    rm(Temp2, Name)
  } # end ToFile if
  
  #update the Data
  Data <- Temp1[["Data"]]
  # extract the SumGDM
  SumGDM2 <- Temp1[["SumGDM"]]
  SumGDM[ROW, match(names(SumGDM2), names(SumGDM))] <- SumGDM2
  ROW <- ROW +1
  rm(Temp1, Details2, SumGDM2)
  
  ##############################################################################
  #### Step3 - Temporal Only                                                 ###
  ##############################################################################
  Details2 <- Details[Details$Group %in% c("Temporal"),]
  ModName <- "AllSites_Temp"
  
  ModInfo[3, "ModName"]       <- ModName
  ModInfo[3, "Spatial"]       <- 0
  ModInfo[3, "Temporal"]      <- 1
  ModInfo[3, "Spectral"]      <- 0
  ModInfo[3, "Environmental"] <- 0
  
  Temp1 <- SingleGDM(Data    = Data,
                     NewData = NULL,
                     Depen   = Depen,
                     Vars    = c(Vars, Details2[, 4], Details2[, 5]),
                     Method  = Method,
                     geo     = geo,
                     splines = splines,
                     knots   = knots,
                     Run     = Run, 
                     ModName = ModName,
                     verbose = verbose)
  # write Isplines to file
  if(ToFile){
    Temp2 <- Temp1[["ISplines"]]
    Name  <- paste("R", Run, "_",ModName, "_Splines.csv", sep="")
    setwd(ResDir)
    write.table(Temp2, Name, sep=",", row.names=FALSE, col.names=TRUE)
    rm(Temp2, Name)
  } # end ToFile if
  #update the Data
  Data <- Temp1[["Data"]]
  # extract the SumGDM
  SumGDM2 <- Temp1[["SumGDM"]]
  SumGDM[ROW, match(names(SumGDM2), names(SumGDM))] <- SumGDM2
  ROW <- ROW +1
  rm(Temp1, Details2, SumGDM2)
  
  ##############################################################################
  #### Step4 - Spectral Only                                                 ###
  ##############################################################################
  if(NMDS){
    Details2 <- Details[Details$Group %in% c("SpecNMDS"),]
  }
  if(!NMDS){
    Details2 <- Details[Details$Group %in% c("Spectral"),]
  }
  
  
  ModName <- "AllSites_Spec"
  
  ModInfo[4, "ModName"]       <- ModName
  ModInfo[4, "Spatial"]       <- 0
  ModInfo[4, "Temporal"]      <- 0
  ModInfo[4, "Spectral"]      <- 1
  ModInfo[4, "Environmental"] <- 0
  
  Temp1 <- SingleGDM(Data    = Data,
                     NewData = NULL,
                     Depen   = Depen,
                     Vars    = c(Vars, Details2[, 4], Details2[, 5]),
                     Method  = Method,
                     geo     = geo,
                     splines = splines,
                     knots   = knots,
                     Run     = Run, 
                     ModName = ModName,
                     verbose = verbose)
  # write Isplines to file
  if(ToFile){
    Temp2 <- Temp1[["ISplines"]]
    Name  <- paste("R", Run, "_",ModName, "_Splines.csv", sep="")
    setwd(ResDir)
    write.table(Temp2, Name, sep=",", row.names=FALSE, col.names=TRUE)
    rm(Temp2, Name)
  } # end ToFile if
  #update the Data
  Data <- Temp1[["Data"]]
  # extract the SumGDM
  SumGDM2 <- Temp1[["SumGDM"]]
  SumGDM[ROW, match(names(SumGDM2), names(SumGDM))] <- SumGDM2
  ROW <- ROW +1
  rm(Temp1, Details2, SumGDM2)
  
  ##############################################################################
  #### Step5 - Environmental Only                                            ###
  ##############################################################################
  
  if(NMDS){
    Details2 <- Details[Details$Group %in% c("EnviNMDS"),]
  }
  if(!NMDS){
    Details2 <- Details[Details$Group %in% c("Environmental"),]
  }
  
  ModName <- "AllSites_Envi"
  
  ModInfo[5, "ModName"]       <- ModName
  ModInfo[5, "Spatial"]       <- 0
  ModInfo[5, "Temporal"]      <- 0
  ModInfo[5, "Spectral"]      <- 0
  ModInfo[5, "Environmental"] <- 1
  
  Temp1 <- SingleGDM(Data    = Data,
                     NewData = NULL,
                     Depen   = Depen,
                     Vars    = c(Vars, Details2[, 4], Details2[, 5]),
                     Method  = Method,
                     geo     = geo,
                     splines = splines,
                     knots   = knots,
                     Run     = Run, 
                     ModName = ModName,
                     verbose = verbose)
  # write Isplines to file
  if(ToFile){
    Temp2 <- Temp1[["ISplines"]]
    Name  <- paste("R", Run, "_",ModName, "_Splines.csv", sep="")
    setwd(ResDir)
    write.table(Temp2, Name, sep=",", row.names=FALSE, col.names=TRUE)
    rm(Temp2, Name)
  } # end ToFile if
  #update the Data
  Data <- Temp1[["Data"]]
  # extract the SumGDM
  SumGDM2 <- Temp1[["SumGDM"]]
  SumGDM[ROW, match(names(SumGDM2), names(SumGDM))] <- SumGDM2
  ROW <- ROW +1
  rm(Temp1, Details2, SumGDM2)
  
  ##############################################################################
  #### Step6 - Spatial + Temporal                                            ###
  ##############################################################################
  Details2 <- Details[Details$Group %in% c("Spatial", "Temporal"), ]
  ModName <- "AllSites_SpatTemp"
  
  ModInfo[6, "ModName"]       <- ModName
  ModInfo[6, "Spatial"]       <- 1
  ModInfo[6, "Temporal"]      <- 1
  ModInfo[6, "Spectral"]      <- 0
  ModInfo[6, "Environmental"] <- 0
  
  Temp1 <- SingleGDM(Data    = Data,
                     NewData = NULL,
                     Depen   = Depen,
                     Vars    = c(Vars, Details2[, 4], Details2[, 5]),
                     Method  = Method,
                     geo     = geo,
                     splines = splines,
                     knots   = knots,
                     Run     = Run, 
                     ModName = ModName,
                     verbose = verbose)
  # write Isplines to file
  if(ToFile){
    Temp2 <- Temp1[["ISplines"]]
    Name  <- paste("R", Run, "_",ModName, "_Splines.csv", sep="")
    setwd(ResDir)
    write.table(Temp2, Name, sep=",", row.names=FALSE, col.names=TRUE)
    rm(Temp2, Name)
  } # end ToFile if
  #update the Data
  Data <- Temp1[["Data"]]
  # extract the SumGDM
  SumGDM2 <- Temp1[["SumGDM"]]
  SumGDM[ROW, match(names(SumGDM2), names(SumGDM))] <- SumGDM2
  ROW <- ROW +1
  rm(Temp1, Details2, SumGDM2)
  
  ##############################################################################
  #### Step7 - Spatial + Spectral                                            ###
  ##############################################################################
  if(NMDS){
    Details2 <- Details[Details$Group %in% c("Spatial", 
                                             "SpecNMDS"),]
  }
  if(!NMDS){
    Details2 <- Details[Details$Group %in% c("Spatial", 
                                             "Spectral"),]
  }
 
  ModName <- "AllSites_SpatSpec"
  
  ModInfo[7, "ModName"]       <- ModName
  ModInfo[7, "Spatial"]       <- 1
  ModInfo[7, "Temporal"]      <- 0
  ModInfo[7, "Spectral"]      <- 1
  ModInfo[7, "Environmental"] <- 0
  
  Temp1 <- SingleGDM(Data    = Data,
                     NewData = NULL,
                     Depen   = Depen,
                     Vars    = c(Vars, Details2[, 4], Details2[, 5]),
                     Method  = Method,
                     geo     = geo,
                     splines = splines,
                     knots   = knots,
                     Run     = Run, 
                     ModName = ModName,
                     verbose = verbose)
  # write Isplines to file
  if(ToFile){
    Temp2 <- Temp1[["ISplines"]]
    Name  <- paste("R", Run, "_",ModName, "_Splines.csv", sep="")
    setwd(ResDir)
    write.table(Temp2, Name, sep=",", row.names=FALSE, col.names=TRUE)
    rm(Temp2, Name)
  } # end ToFile if
  #update the Data
  Data <- Temp1[["Data"]]
  # extract the SumGDM
  SumGDM2 <- Temp1[["SumGDM"]]
  SumGDM[ROW, match(names(SumGDM2), names(SumGDM))] <- SumGDM2
  ROW <- ROW +1
  rm(Temp1, Details2, SumGDM2)
  
  ##############################################################################
  #### Step8 - Spatial + Environmental                                       ###
  ##############################################################################
  if(NMDS){
    Details2 <- Details[Details$Group %in% c("Spatial", 
                                             "EnviNMDS"),]
  }
  if(!NMDS){
    Details2 <- Details[Details$Group %in% c("Spatial", 
                                             "Environmental"),]
  }
  
  ModName <- "AllSites_SpatEnvi"
  
  ModInfo[8, "ModName"]       <- ModName
  ModInfo[8, "Spatial"]       <- 1
  ModInfo[8, "Temporal"]      <- 0
  ModInfo[8, "Spectral"]      <- 0
  ModInfo[8, "Environmental"] <- 1
  
  Temp1 <- SingleGDM(Data    = Data,
                     NewData = NULL,
                     Depen   = Depen,
                     Vars    = c(Vars, Details2[, 4], Details2[, 5]),
                     Method  = Method,
                     geo     = geo,
                     splines = splines,
                     knots   = knots,
                     Run     = Run, 
                     ModName = ModName,
                     verbose = verbose)
  # write Isplines to file
  if(ToFile){
    Temp2 <- Temp1[["ISplines"]]
    Name  <- paste("R", Run, "_",ModName, "_Splines.csv", sep="")
    setwd(ResDir)
    write.table(Temp2, Name, sep=",", row.names=FALSE, col.names=TRUE)
    rm(Temp2, Name)
  } # end ToFile if
  #update the Data
  Data <- Temp1[["Data"]]
  # extract the SumGDM
  SumGDM2 <- Temp1[["SumGDM"]]
  SumGDM[ROW, match(names(SumGDM2), names(SumGDM))] <- SumGDM2
  ROW <- ROW +1
  rm(Temp1, Details2, SumGDM2)
  
  ##############################################################################
  #### Step9 - Temporal + Spectral                                           ###
  ##############################################################################
  if(NMDS){
    Details2 <- Details[Details$Group %in% c("Temporal", 
                                             "SpecNMDS"),]
  }
  if(!NMDS){
    Details2 <- Details[Details$Group %in% c("Temporal", 
                                             "Spectral"),]
  }
  
  ModName <- "AllSites_TempSpec"
  
  ModInfo[9, "ModName"]       <- ModName
  ModInfo[9, "Spatial"]       <- 0
  ModInfo[9, "Temporal"]      <- 1
  ModInfo[9, "Spectral"]      <- 1
  ModInfo[9, "Environmental"] <- 0
  
  Temp1 <- SingleGDM(Data    = Data,
                     NewData = NULL,
                     Depen   = Depen,
                     Vars    = c(Vars, Details2[, 4], Details2[, 5]),
                     Method  = Method,
                     geo     = geo,
                     splines = splines,
                     knots   = knots,
                     Run     = Run, 
                     ModName = ModName,
                     verbose = verbose)
  # write Isplines to file
  if(ToFile){
    Temp2 <- Temp1[["ISplines"]]
    Name  <- paste("R", Run, "_",ModName, "_Splines.csv", sep="")
    setwd(ResDir)
    write.table(Temp2, Name, sep=",", row.names=FALSE, col.names=TRUE)
    rm(Temp2, Name)
  } # end ToFile if
  #update the Data
  Data <- Temp1[["Data"]]
  # extract the SumGDM
  SumGDM2 <- Temp1[["SumGDM"]]
  SumGDM[ROW, match(names(SumGDM2), names(SumGDM))] <- SumGDM2
  ROW <- ROW +1
  rm(Temp1, Details2, SumGDM2)
  
  ##############################################################################
  #### Step10 - Temporal + Environmental                                     ###
  ##############################################################################
  if(NMDS){
    Details2 <- Details[Details$Group %in% c("Temporal", 
                                             "EnviNMDS"),]
  }
  if(!NMDS){
    Details2 <- Details[Details$Group %in% c("Temporal", 
                                             "Environmental"),]
  }
  
  ModName <- "AllSites_TempEnvi"
  
  ModInfo[10, "ModName"]       <- ModName
  ModInfo[10, "Spatial"]       <- 0
  ModInfo[10, "Temporal"]      <- 1
  ModInfo[10, "Spectral"]      <- 0
  ModInfo[10, "Environmental"] <- 1
  
  Temp1 <- SingleGDM(Data    = Data,
                     NewData = NULL,
                     Depen   = Depen,
                     Vars    = c(Vars, Details2[, 4], Details2[, 5]),
                     Method  = Method,
                     geo     = geo,
                     splines = splines,
                     knots   = knots,
                     Run     = Run, 
                     ModName = ModName,
                     verbose = verbose)
  # write Isplines to file
  if(ToFile){
    Temp2 <- Temp1[["ISplines"]]
    Name  <- paste("R", Run, "_",ModName, "_Splines.csv", sep="")
    setwd(ResDir)
    write.table(Temp2, Name, sep=",", row.names=FALSE, col.names=TRUE)
    rm(Temp2, Name)
  } # end ToFile if
  #update the Data
  Data <- Temp1[["Data"]]
  # extract the SumGDM
  SumGDM2 <- Temp1[["SumGDM"]]
  SumGDM[ROW, match(names(SumGDM2), names(SumGDM))] <- SumGDM2
  ROW <- ROW +1
  rm(Temp1, Details2, SumGDM2)
  
  ##############################################################################
  #### Step11 - Spectral + Environmental                                     ###
  ##############################################################################
  if(NMDS){
    Details2 <- Details[Details$Group %in% c("SpecNMDS", 
                                             "EnviNMDS"),]
  }
  if(!NMDS){
    Details2 <- Details[Details$Group %in% c("Spectral", 
                                             "Environmental"),]
  }
  
  ModName <- "AllSites_SpecEnvi"
  
  ModInfo[11, "ModName"]       <- ModName
  ModInfo[11, "Spatial"]       <- 0
  ModInfo[11, "Temporal"]      <- 0
  ModInfo[11, "Spectral"]      <- 1
  ModInfo[11, "Environmental"] <- 1
  
  Temp1 <- SingleGDM(Data    = Data,
                     NewData = NULL,
                     Depen   = Depen,
                     Vars    = c(Vars, Details2[, 4], Details2[, 5]),
                     Method  = Method,
                     geo     = geo,
                     splines = splines,
                     knots   = knots,
                     Run     = Run, 
                     ModName = ModName,
                     verbose = verbose)
  # write Isplines to file
  if(ToFile){
    Temp2 <- Temp1[["ISplines"]]
    Name  <- paste("R", Run, "_",ModName, "_Splines.csv", sep="")
    setwd(ResDir)
    write.table(Temp2, Name, sep=",", row.names=FALSE, col.names=TRUE)
    rm(Temp2, Name)
  } # end ToFile if
  #update the Data
  Data <- Temp1[["Data"]]
  # extract the SumGDM
  SumGDM2 <- Temp1[["SumGDM"]]
  SumGDM[ROW, match(names(SumGDM2), names(SumGDM))] <- SumGDM2
  ROW <- ROW +1
  rm(Temp1, Details2, SumGDM2)
  
  ##############################################################################
  #### Step12 - Spatial + Temporal + Spectral                                ###
  ##############################################################################
  if(NMDS){
    Details2 <- Details[Details$Group %in% c("Spatial", 
                                             "Temporal", 
                                             "SpecNMDS"),]
  }
  if(!NMDS){
    Details2 <- Details[Details$Group %in% c("Spatial", 
                                             "Temporal", 
                                             "Spectral"),]
  }
  
  ModName <- "AllSites_SpatTempSpec"
  
  ModInfo[12, "ModName"]       <- ModName
  ModInfo[12, "Spatial"]       <- 1
  ModInfo[12, "Temporal"]      <- 1
  ModInfo[12, "Spectral"]      <- 1
  ModInfo[12, "Environmental"] <- 0
  
  Temp1 <- SingleGDM(Data    = Data,
                     NewData = NULL,
                     Depen   = Depen,
                     Vars    = c(Vars, Details2[, 4], Details2[, 5]),
                     Method  = Method,
                     geo     = geo,
                     splines = splines,
                     knots   = knots,
                     Run     = Run, 
                     ModName = ModName,
                     verbose = verbose)
  # write Isplines to file
  if(ToFile){
    Temp2 <- Temp1[["ISplines"]]
    Name  <- paste("R", Run, "_",ModName, "_Splines.csv", sep="")
    setwd(ResDir)
    write.table(Temp2, Name, sep=",", row.names=FALSE, col.names=TRUE)
    rm(Temp2, Name)
  } # end ToFile if
  #update the Data
  Data <- Temp1[["Data"]]
  # extract the SumGDM
  SumGDM2 <- Temp1[["SumGDM"]]
  SumGDM[ROW, match(names(SumGDM2), names(SumGDM))] <- SumGDM2
  ROW <- ROW +1
  rm(Temp1, Details2, SumGDM2)
  
  ##############################################################################
  #### Step13 - Spatial + Temporal + Environmental                           ###
  ##############################################################################
  if(NMDS){
    Details2 <- Details[Details$Group %in% c("Spatial", 
                                             "Temporal", 
                                             "EnviNMDS"),]
  }
  if(!NMDS){
    Details2 <- Details[Details$Group %in% c("Spatial", 
                                             "Temporal", 
                                             "Environmental"),]
  }
  
  ModName <- "AllSites_SpatTempEnvi"
  
  ModInfo[13, "ModName"]       <- ModName
  ModInfo[13, "Spatial"]       <- 1
  ModInfo[13, "Temporal"]      <- 1
  ModInfo[13, "Spectral"]      <- 0
  ModInfo[13, "Environmental"] <- 1
  
  Temp1 <- SingleGDM(Data    = Data,
                     NewData = NULL,
                     Depen   = Depen,
                     Vars    = c(Vars, Details2[, 4], Details2[, 5]),
                     Method  = Method,
                     geo     = geo,
                     splines = splines,
                     knots   = knots,
                     Run     = Run, 
                     ModName = ModName,
                     verbose = verbose)
  # write Isplines to file
  if(ToFile){
    Temp2 <- Temp1[["ISplines"]]
    Name  <- paste("R", Run, "_",ModName, "_Splines.csv", sep="")
    setwd(ResDir)
    write.table(Temp2, Name, sep=",", row.names=FALSE, col.names=TRUE)
    rm(Temp2, Name)
  } # end ToFile if
  #update the Data
  Data <- Temp1[["Data"]]
  # extract the SumGDM
  SumGDM2 <- Temp1[["SumGDM"]]
  SumGDM[ROW, match(names(SumGDM2), names(SumGDM))] <- SumGDM2
  ROW <- ROW +1
  rm(Temp1, Details2, SumGDM2)
  
  ##############################################################################
  #### Step14 - Spatial + Spectral + Environmental                           ###
  ##############################################################################
  if(NMDS){
    Details2 <- Details[Details$Group %in% c("Spatial", 
                                             "SpecNMDS", 
                                             "EnviNMDS"),]
  }
  if(!NMDS){
    Details2 <- Details[Details$Group %in% c("Spatial", 
                                             "Spectral", 
                                             "Environmental"),]
  }
  
  ModName <- "AllSites_SpatSpecEnvi"
  
  ModInfo[14, "ModName"]       <- ModName
  ModInfo[14, "Spatial"]       <- 1
  ModInfo[14, "Temporal"]      <- 0
  ModInfo[14, "Spectral"]      <- 1
  ModInfo[14, "Environmental"] <- 1
  
  Temp1 <- SingleGDM(Data    = Data,
                     NewData = NULL,
                     Depen   = Depen,
                     Vars    = c(Vars, Details2[, 4], Details2[, 5]),
                     Method  = Method,
                     geo     = geo,
                     splines = splines,
                     knots   = knots,
                     Run     = Run, 
                     ModName = ModName,
                     verbose = verbose)
  # write Isplines to file
  if(ToFile){
    Temp2 <- Temp1[["ISplines"]]
    Name  <- paste("R", Run, "_",ModName, "_Splines.csv", sep="")
    setwd(ResDir)
    write.table(Temp2, Name, sep=",", row.names=FALSE, col.names=TRUE)
    rm(Temp2, Name)
  } # end ToFile if
  #update the Data
  Data <- Temp1[["Data"]]
  # extract the SumGDM
  SumGDM2 <- Temp1[["SumGDM"]]
  SumGDM[ROW, match(names(SumGDM2), names(SumGDM))] <- SumGDM2
  ROW <- ROW +1
  rm(Temp1, Details2, SumGDM2)
  
  ##############################################################################
  #### Step15 - Temporal + Spectral + Environmental                          ###
  ##############################################################################
  if(NMDS){
    Details2 <- Details[Details$Group %in% c("Temporal", 
                                             "SpecNMDS", 
                                             "EnviNMDS"),]
  }
  if(!NMDS){
    Details2 <- Details[Details$Group %in% c("Temporal", 
                                             "Spectral", 
                                             "Environmental"),]
  }
  
  ModName <- "AllSites_TempSpecEnvi"
  
  ModInfo[15, "ModName"]       <- ModName
  ModInfo[15, "Spatial"]       <- 0
  ModInfo[15, "Temporal"]      <- 1
  ModInfo[15, "Spectral"]      <- 1
  ModInfo[15, "Environmental"] <- 1
  
  Temp1 <- SingleGDM(Data    = Data,
                     NewData = NULL,
                     Depen   = Depen,
                     Vars    = c(Vars, Details2[, 4], Details2[, 5]),
                     Method  = Method,
                     geo     = geo,
                     splines = splines,
                     knots   = knots,
                     Run     = Run, 
                     ModName = ModName,
                     verbose = verbose)
  # write Isplines to file
  if(ToFile){
    Temp2 <- Temp1[["ISplines"]]
    Name  <- paste("R", Run, "_",ModName, "_Splines.csv", sep="")
    setwd(ResDir)
    write.table(Temp2, Name, sep=",", row.names=FALSE, col.names=TRUE)
    rm(Temp2, Name)
  } # end ToFile if
  #update the Data
  Data <- Temp1[["Data"]]
  # extract the SumGDM
  SumGDM2 <- Temp1[["SumGDM"]]
  SumGDM[ROW, match(names(SumGDM2), names(SumGDM))] <- SumGDM2
  ROW <- ROW +1
  rm(Temp1, Details2, SumGDM2)
  
  ##############################################################################
  #### Step16 - Return List                                                  ###
  ##############################################################################
  ReturnList <- list(Data    = Data,
                     SumGDM  = SumGDM,
                     ModInfo = ModInfo)
  return(ReturnList)
} # End main function

