################################################################################
### Name:     AreaWeigSD.R                                                   ###
### aim:                                                                     ###
### Calcualte for each site the  SD over all objects, weighted by            ###
### the object's area.                                                       ###
###                                                                          ###
### Input:                                                                   ###
### Data    - Dataframe with the required data.                              ### 
### SiteCol - Integer, the column number with the site IDs.                  ###
### Areas   - Integer, column number with object area to be used as          ###
###            weights when estimating mean and SD.                          ###
### Vars    - Vector of integers, the column number of variables to          ###
###           calcualte weighted mean and SD for.                            ###
### verbose - Logical, should progress be printed on consule.                ###
###                                                                          ###
### Procedure:                                                               ###
### For each site -                                                          ###
### 1. subset data.                                                          ###
### 2. calcualte relative areas as weights.                                  ###
### 3. for each variable in Vars:                                            ###
###      3.a. Calcualte weighted SD using the funtion outlined first in:     ###
#  http://www.itl.nist.gov/div898/software/dataplot/refman2/ch2/weightsd.pdf   #
###                                                                          ###
###                                                                          ###
### Output:                                                                  ###
### Dataframe with a row for each site. First column in the Sites IDs. For   ###
### each Vars a columns one with the weighted SD.                            ###
###                                                                          ###
### Author: Yoni Gavish <gavishyoni@gmail.com>                               ###
### Date:   21 Sep. 2016                                                     ###
################################################################################


AreaWeigSD = function(Data,
                      SiteCol,
                      Areas,
                      Vars,
                      verbose = T){
  
  if(verbose){
    cat(paste("##----- AreaWeigSD -->     Vars:  ",
              paste(names(Data)[Vars], collapse=", "),
              "\n", 
              sep=""))
    cat(paste("## Site --> "))
  }
  
  
  # create the Output data frame
  DataOut  <- data.frame(Site = unique(Data[, SiteCol]))
  #DataOut[, paste(names(Data)[Vars], "_Mean", sep="")] <- rep(NA, nrow(DataOut))
  DataOut[, paste(names(Data)[Vars], "_SD", sep="")]   <- rep(NA, nrow(DataOut))
  
  for(i in 1:nrow(DataOut)){
    
    if(verbose){cat(paste(DataOut[i, 1], " --> ", sep=""))}
    
    # subset site's data
    Data1              <- Data[Data[, SiteCol]== DataOut[i, 1], c(Areas, Vars)]
    Data1[, "RelArea"] <- Data1[, 1]/sum(Data1[, 1])
    
    # work on each Vars
    for(j in 1:length(Vars)){
      # names in DataOut
      #Name1             <- paste(names(Data)[Vars[j]], "_Mean", sep="")
      Name2             <- paste(names(Data)[Vars[j]], "_SD", sep="")
      
      # weighted Vector + mean
      Vec               <- Data1[, j+1] * Data1[, "RelArea"]
      #DataOut[i, Name1] <- sum(Vec)
      
      # Weighted SD, based on:
      # http://www.itl.nist.gov/div898/software/dataplot/refman2/ch2/weightsd.pdf
      Temp1             <- sum(Data1[, "RelArea"]*(Data1[, j+1] - sum(Vec))^2)
      Temp2             <- (length(Vec) - 1 ) / length(Vec)
      DataOut[i, Name2] <- sqrt(Temp1/Temp2)
      rm(Name2, Vec, Temp1, Temp2)
    } # end Var loop
    
    rm(j, Data1) 
  } # end the Dataout loop
  rm(i)
  if(verbose){
    cat(paste("Done\n"))
  }
  
  return(DataOut)
} # End main function



