################################################################################
### Name:     CoverBC.R                                                      ###
### aim:                                                                     ###
### Calucalte the Bray-Curtis dissimilarity based on the relative cover of   ###
### different classes, as weighted by the area of the objects.               ###
###                                                                          ###
### Input:                                                                   ###
### DataBC  - Dataframe, with the Objects, the sites, the objects area and   ###
###           the categorical varaibles.                                     ###
### SiteCol - Integer, coulmn number with the site IDs.                      ###
### Strat   - Integer, column number with the cateogrical variable on which  ###
###           the BRay-curtis distance will be based.                        ###
### Areas   - Integer, column number with object area to be used as          ###
###           weights when estimating cover.                                 ###
### Pairs   - Dataframe with the pairs of site.                              ###
### verbose - Logical, should progress be printed on consule.                ###
###                                                                          ###
### Process:                                                                 ###
### 1. the relative cover of each factor level is calcualted in each site.   ###
### 2. the Bray-Curtis distances are calcualted for each pair of sites.      ###
###                                                                          ###
### Output:                                                                  ###
### Pairs dataframe with additional column with the bray-curtis distances.   ###
###                                                                          ###
### Author: Yoni Gavish <gavishyoni@gmail.com>                               ###
### Date:   20 Sep. 2016                                                     ###
################################################################################



CoverBC = function(DataBC, 
                   SiteCol,
                   Strat,
                   Areas,
                   Pairs,
                   verbose = T){
  ##############################################################################
  ### required packages                                                      ###
  ##############################################################################
  #require(randomForest)
  #require(caret)
  #require(StatMatch)     # the gower similarity
  #require(psych)         # descriptive statistics
  require(vegan)          # for the bray-curtis dissimilarity
  
  ##############################################################################
  ### Step 1: EStimate relative cover in each site                           ###
  ##############################################################################
  if(verbose){
    cat(paste("##----- CoverBC: ", names(DataBC[Strat]), "\n" ,sep =""))
  }
  
  HabLev <- levels(DataBC[, Strat])
  
  # create the relative cover data frame
  RelCov           <- data.frame(Site = unique(Pairs[,2]))
  RelCov[, HabLev] <- rep(0, nrow(RelCov))
  if(verbose){cat(paste("Exrtracting relative covers --> "))}
  
  for(i in 1:nrow(RelCov)){
    # subset data
    Data <- DataBC[DataBC[, SiteCol] == RelCov[i, 1], c(Strat, Areas)]
    names(Data) <- c("Strat", "Area")
    Temp1 <- aggregate(Area ~ Strat, Data, sum, drop= F)
    Temp1[, "RelArea"] <- Temp1[, 2]/sum(Temp1[, 2])
    for(k in 1:nrow(Temp1)){
      Col <- match(Temp1[k, 1], names(RelCov))
      RelCov[i, Col] <- Temp1[k, 3]
      rm(Col)
    } # End Temp1 loop
    rm(Data, Temp1)
  } # end RelCov loop
  
  ##############################################################################
  ### Step 2: Calculate Bray-Curtis dissimilarity                            ###
  ##############################################################################
  
  if(verbose){cat(paste("Bray-Curtis distances --> "))}
  DisBC  <- vegdist(RelCov[, 2:ncol(RelCov)], method="bray", binary=FALSE, diag=T, upper=T, na.rm = FALSE) 
  DisBC <- as.matrix(DisBC)
  colnames(DisBC) <- RelCov[,1 ] 
  rownames(DisBC) <- RelCov[,1 ]

  for (i in 1:nrow(Pairs)){
    Site1 <- match(Pairs[i,2], rownames(DisBC))
    Site2 <- match(Pairs[i,3], rownames(DisBC))
    Pairs[i, paste("Dis_", names(DataBC)[Strat], sep="")] <- DisBC[Site1, Site2]
    rm(Site1, Site2)
  } # end pairs loop
  if(verbose){
    cat(paste("Done"))
    cat(paste("\n##-----------------------------------------------------##\n"))}
  return(Pairs)
} # end main function




