################################################################################
### Name:     GavishWorkFlow.R                                               ###
### aim:                                                                     ###
### conduct the distance decay analysis with class = 9 and All vars for      ###
### spectral and Environmental.                                              ###
###                                                                          ###
### Input:                                                                   ###
### 1. The basic DataFull dataframe.                                         ### 
### 2. the different functions to source.                                    ###
### 3. the output of the data preperation procedure.                         ### 
### 4. various controls to set below.                                        ###
###                                                                          ###
### Procedure:                                                               ###
### For each Run -                                                           ###
### 1.  Read from file                                                       ###
### 2.  Run all RF models and write to file                                  ###
### 3.  Run all the GDMs with all the sites                                  ###
### 4.  Run all the GDMs with each site left out                             ###
### 5.  write SumGDM and Pairsfull to file and update the general versions   ###
### 6.  Predict kappa for each site: MeanLocObs                              ###
### 7.  Predict kappa for each site: MeanLocEqu                              ###
### 8.  Predict kappa for each site: MeanLocPre                              ###
### 9.  Predict kappa for each site: MaxLocPre (self excluded)               ###
### 10. Write Sites to file and update the general version                   ###
###                                                                          ###
### Output:                                                                  ###
### 1. Output from each run is written to file.                              ###
### 2. Output from all Runs for the Pairs, Sites, SumGDm and ModInfo are     ###
###    binded and saved to file.                                             ###
### 3. Check additional information in each function.                        ###
###                                                                          ###
### Notes:                                                                   ###
### 1. This workflow contains additional steps of analysis that were not     ###
### included in the manuscript. However, we have not removed them from here  ###
### to ensure reproducability of our results. In addition, it will require   ###
### re-running the analysis to ensure the code is still operational.         ###
### 2. the steps added:                                                      ###
###    2.1 : a global model - the kappa when training a model in 9 sites and ###
###          predicting in 1 sites that was set aside.                       ###
###    2.2 : MeanLocObs, MeanLocEqu, MeanLocPre - 3 kappas produced by       ###
###          averaging the probabilities of all local models for a given     ###
###          site that is set aside. For example, if site 1 is set a side,   ###
###          the 9 local models trained in sites 2-10 will be averaged.      ###
###          Averging is done at the pixel level by taking the proporiton of ###
###          votes for each class and multiplying it by weight given to a    ###
###          given model. the 3 option weight the models differntly. the     ###
###          first used the observed kappa of the models in site 1 as        ###
###          weights. the second gives equal weights for all the models.     ###
###          the 3rd used the predicted kappa as weights.                    ###
###    2.3 : MaxLocPre - the kappa of the sites predcited to have the        ###
###          highest value in a novel target site based on the GDM.          ###
### 3. Running the entire analysis can take considerable time. To test the   ###
### code the NTree value in the controls section should be set to a much     ###
### lower value (e.g., 100).                                                 ###
###                                                                          ###
### Author: Yoni Gavish <gavishyoni@gmail.com>                               ###
###                                                                          ###
### Versions:                                                                ### 
###  1. 26 Sep. 2016 - original code used in the analysis.                   ###
###  2. 03 Aug. 2017 - slight edits for the Dryad version.                   ### 
################################################################################


################################################################################
### Files Directories -------------------------------------------------------###
################################################################################

# Input directory where all the input files and functions are stored 
InpDir <- "Your path here"

# Outout directory where all the output files are written
ResDir <- "Your path here"

################################################################################
### Read Data ---------------------------------------------------------------###
################################################################################
setwd(InpDir)
DataFull <- read.csv("DataFull.csv")
VarInfo  <- read.csv("DataFullVarInfo.csv")

################################################################################
### Controls ----------------------------------------------------------------###
################################################################################

RUNS        <- c(1,10)   # the start and end for Run
Truth       <- 6         # culumn number with the ground truth data
TruthName   <- "C9"
NMDS        <- F         # if T, the nMDS results for Spectral and Environmental varaibles are used in the GDMs
ToFile      <- T         # if T, results are written to file
ObjectID    <- 2         # culumn number with the object Id
SiteCol     <- 3         # culumn number with the site ID
Ntree       <- 10000     # RF control - number of classifcation trees in each run
verbose     <- T
# culumn numbers of the explantroy variables for the RF analyses
RFVars      <- VarInfo[VarInfo$RF == 1, "ColNum"] 
# culumn number 'in PairsFull' with the 6 basic GDM input variables (distance, weights and 4 coordinates)
GDMVars     <- c(4:9)  

################################################################################
### Wrapper around all Runs -------------------------------------------------###
################################################################################

for(Run in RUNS[1]:RUNS[2]){
  
  if(verbose){
    cat(paste("\n\n##--------------------------------------------------------------------------------##\n"))
    cat(paste(    "##--------------------------------------------------------------------------------##\n"))
    cat(paste(    "##--------------------------------------------------------------------------------##\n"))
    cat(paste(    "## Run:", Run, "\n", sep=""))
    cat(paste(    "##--------------------------------------------------------------------------------##\n"))
    cat(paste(    "##--------------------------------------------------------------------------------##\n"))
    cat(paste(    "##--------------------------------------------------------------------------------##\n"))
  }
  
  
  ##############################################################################
  ### 1- Read from file + source functions ----------------------------------###
  ##############################################################################
  
  setwd(InpDir)
  DataRF     <- read.csv("DataFull.csv")
  PairsFull  <- read.csv("PairsFull.csv")
  Details    <- read.csv("PairsDetails.csv")
  source("RunAllRF.R")
  source("AllFullGDM.R")
  source("SitesGDM.R")
  source("WeiPropFull.R")
  
  ##############################################################################
  ### 2- Run all RF models and write to file  -------------------------------###
  ##############################################################################
  
  RF <- RunAllRF(DataRF   = DataRF,
                 Truth    = Truth ,
                 Vars     = RFVars,
                 ObjectID = ObjectID,
                 SiteCol  = SiteCol,
                 Pairs    = PairsFull[, c(1:3)],
                 ResDir   = ResDir,
                 Run      = Run,
                 Ntree    = Ntree,
                 verbose  = verbose)
  
  AllRF  <- RF[["AllRF"]]
  Pairs2 <- RF[["Pairs"]]
  Sites  <- RF[["Sites"]]
  
  # add the observed kappa to PairsFull
  PairsFull$ObsKappa <- Pairs2$ObsKappa
  ObsKappaCol        <- match("ObsKappa", names(PairsFull))
  rm(Pairs2)
  
  ##############################################################################
  ### 3- Run all the GDMs with all the sites  -------------------------------###
  ##############################################################################
  
  if(verbose){
    cat(paste("\n##-----------------------------------------------------##\n"))
    cat(paste(  "## Step 4 - Run the GDMs with all the sites:           ##"))
  }
  
  FullGDM<- AllFullGDM(Data    = PairsFull,
                       Details = Details,
                       Depen   = ObsKappaCol,
                       Vars    = GDMVars,
                       Method  = "CompStan11",
                       NMDS    = NMDS,
                       geo     = F,
                       splines = NULL,
                       knots   = NULL,
                       Run     = Run,
                       FunDir  = InpDir,
                       ResDir  = ResDir,
                       ToFile  = ToFile,
                       verbose = verbose)
  
  # extract data
  PairsFull <- FullGDM$Data
  SumGDM    <- FullGDM$SumGDM
  ModInfo   <- FullGDM$ModInfo
  # add the run column 
  ModInfo <- cbind(rep(Run, nrow(ModInfo)), ModInfo)
  names(ModInfo)[1] <- "Run"
  
  rm(FullGDM)
  
  ##############################################################################
  ### 4- Run all the GDMs with each site left out  --------------------------###
  ##############################################################################
  
  if(verbose){
    cat(paste("\n##-----------------------------------------------------##\n"))
    cat(paste(  "## Step 5 - Run the GDMs with each site left out:      ##"))
  }
  
  if(NMDS){
    Details2 <- Details[Details$Group %in% c("Spatial", 
                                             "Temporal", 
                                             "SpecNMDS", 
                                             "EnviNMDS"),]
  }
  if(!NMDS){
    Details2 <- Details[Details$Group %in% c("Spatial", 
                                             "Temporal", 
                                             "Spectral", 
                                             "Environmental"),]
  }
  
  SetAsideGDM <- SitesGDM(Data    = PairsFull,
                          Depen   = ObsKappaCol,
                          Vars    = c(GDMVars, Details2[, 4], Details2[, 5]),
                          Method  = "CompStan11",
                          geo     = F,
                          splines = NULL,
                          knots   = NULL,
                          Run     = Run,
                          FunDir  = InpDir,
                          ResDir  = ResDir,
                          ToFile  = ToFile,
                          verbose = verbose)
  
  PairsFull  <- SetAsideGDM[["Data"]]
  Temp1 <- SetAsideGDM[["SumGDM"]] 
  # check the names match and return an error if they don't
  if(!all(names(Temp1) == names(SumGDM))){
    stop(paste("\nWorkFlow - Step4: cannot match all names of SumGDM\n"))
  }
  
  SumGDM <- rbind(SumGDM, Temp1)
  rm(Details2, SetAsideGDM, Temp1)
  
  ##############################################################################
  ### 5- write SumGDM and Pairsfull to file and update the general versions -###
  ##############################################################################
  
  # add the Run to PairsFull
  PairsFull <- cbind(rep(Run, nrow(PairsFull)), PairsFull)
  names(PairsFull)[1] <- "Run"
  
  if(ToFile){
    Name1 <- paste("R", Run, "_PairsFull.csv", sep="")
    Name2 <- paste("R", Run, "_SumGDM.csv", sep="")
    Name3 <- paste("R", Run, "_ModInfo.csv", sep="")
    setwd(ResDir)
    write.table(PairsFull, Name1, sep=",", row.names=FALSE, col.names=TRUE)
    write.table(SumGDM,    Name2, sep=",", row.names=FALSE, col.names=TRUE)
    write.table(ModInfo,    Name3, sep=",", row.names=FALSE, col.names=TRUE)
    rm(Name1, Name2, Name3)
  }
  
  if(Run == RUNS[1]){
    PairsAllRuns <- PairsFull
    SumGDMAllRuns <- SumGDM
    ModInfoAllRuns <- ModInfo
  }
  
  if(Run != RUNS[1]){
    PairsAllRuns   <- rbind(PairsAllRuns, PairsFull)
    SumGDMAllRuns  <- rbind(SumGDMAllRuns, SumGDM)
    ModInfoAllRuns <- rbind(ModInfoAllRuns, ModInfo)
  }
  
  ##############################################################################
  ### 6- Kappas per site: MeanLocObs ----------------------------------------###
  ##############################################################################
  
  if(verbose){
    cat(paste("\n##-----------------------------------------------------##\n"))
    cat(paste(  "## Step 6 - Kappas per site: MeanLocObs:               ##"))
  }
  
  # Prepare the input with the observed kappa
  Temp1       <- PairsFull[, c(2:4)]
  Temp1$NAME  <- Sites[match(Temp1$InputID, Sites[,1]), "NAME" ]
  Temp1$Kappa <- PairsFull[, ObsKappaCol]
  
  # run the WeiPropFull function to predcite the mean proportion of votes and expected kappa
  Temp2 <- WeiPropFull(AllRF      = AllRF, 
                       WeiKapFull = Temp1,
                       FunDir     = InpDir,
                       WeiType    = "MeanLocObs",
                       verbose    = verbose)
  
  #names(Out_WeiPropFull)
  AllObj  <- Temp2[["AllObj"]]
  Sites2  <- Temp2[["SiteAll"]]
  
  for (k in 1:nrow(Sites)){
    Sites[k, "MeanLocObs"] <- Sites2[match(Sites[k, 1], Sites2[, 1]), "MeanLocObs"]
  }
  
  #write the AllObj to file
  if(ToFile){
    NAME <- paste("R", Run, "_MeanLocObs_Votes.csv", sep="")
    setwd(ResDir)
    write.table(AllObj, NAME, sep=",", row.names=FALSE, col.names=TRUE)
    rm(NAME)
  }
  rm(Temp1, Temp2, AllObj, Sites2, k)
  
  ##############################################################################
  ### 7- Predict kappa for each site: MeanLocEqu ----------------------------###
  ##############################################################################
  
  if(verbose){
    cat(paste("\n##-----------------------------------------------------##\n"))
    cat(paste(  "## Step 7 - Kappas per site: MeanLocEqu:               ##"))
  }
  # Prepare the input with the observed kappa
  Temp1       <- PairsFull[, c(2:4)]
  Temp1$NAME  <- Sites[match(Temp1$InputID, Sites[,1]), "NAME" ]
  Temp1$Kappa <- 1
  
  # run the WeiPropFull function to predcite the mean proportion of votes and expected kappa
  Temp2 <- WeiPropFull(AllRF      = AllRF, 
                       WeiKapFull = Temp1,
                       FunDir     = InpDir,
                       WeiType    = "MeanLocEqu",
                       verbose    = verbose)
  
  #names(Out_WeiPropFull)
  AllObj  <- Temp2[["AllObj"]]
  Sites2  <- Temp2[["SiteAll"]]
  
  for (k in 1:nrow(Sites)){
    Sites[k, "MeanLocEqu"] <- Sites2[match(Sites[k, 1], Sites2[, 1]), "MeanLocEqu"]
  }
  
  #write the AllObj to file
  if(ToFile){
    NAME <- paste("R", Run, "_MeanLocEqu_Votes.csv", sep="")
    setwd(ResDir)
    write.table(AllObj, NAME, sep=",", row.names=FALSE, col.names=TRUE)
    rm(NAME)
  }
  rm(Temp1, Temp2, AllObj, Sites2, k)
  
  ##############################################################################
  ### 8- Predict kappa for each site: MeanLocPre ----------------------------###
  ##############################################################################
  
  if(verbose){
    cat(paste("\n##-----------------------------------------------------##\n"))
    cat(paste(  "## Step 8 - Kappas per site: MeanLocPre:               ##"))
  }
  # Prepare the input with the observed kappa
  Temp1       <- PairsFull[, c(2:4)]
  Temp1$NAME  <- Sites[match(Temp1$InputID, Sites[,1]), "NAME" ]
  Temp1$Kappa <- PairsFull[, "Kappa_Target"]
  
  # run the WeiPropFull function to predcit the mean proportion of votes and expected kappa
  Temp2 <- WeiPropFull(AllRF      = AllRF, 
                       WeiKapFull = Temp1,
                       FunDir     = InpDir,
                       WeiType    = "MeanLocPre",
                       verbose    = verbose)
  
  #names(Out_WeiPropFull)
  AllObj  <- Temp2[["AllObj"]]
  Sites2  <- Temp2[["SiteAll"]]
  
  for (k in 1:nrow(Sites)){
    Sites[k, "MeanLocPre"] <- Sites2[match(Sites[k, 1], Sites2[, 1]), "MeanLocPre"]
  }
  
  #write the AllObj to file
  if(ToFile){
    NAME <- paste("R", Run, "_MeanLocPre_Votes.csv", sep="")
    setwd(ResDir)
    write.table(AllObj, NAME, sep=",", row.names=FALSE, col.names=TRUE)
    rm(NAME)
  }
  rm(Temp1, Temp2, AllObj, Sites2, k)
  
  ##############################################################################
  ### 9- Predict kappa for each site: MaxLocPre (self excluded) -------------###
  ##############################################################################
  
  if(verbose){
    cat(paste("\n##-----------------------------------------------------##\n"))
    cat(paste(  "## Step 9 - Kappas per site: MaxLocPre:                ##"))
  }
  
  for(k in 1:nrow(Sites)){
    Temp1 <- PairsFull[PairsFull$TargetID == Sites[k, 1], ]
    Temp1 <- Temp1[Temp1$InputID != Sites[k, 1], ]
    #Temp1 <- Temp1[, c(1:10, 105, 151, 152)]
    # add the observd kappa of the site with the highest predicted kappa
    Sites[k, "MaxLocPre"] <- Temp1[match(max(Temp1[, "Kappa_Target"]), 
                                         Temp1[, "Kappa_Target"]), 
                                   "ObsKappa"]
    rm(Temp1)
  }
  rm(k)
  
  
  ##############################################################################
  ### 10- Write Sites to file and update the general version ----------------###
  ##############################################################################
  
  # add the run column 
  Sites <- cbind(rep(Run, nrow(Sites)), Sites)
  names(Sites)[1] <- "Run"
  
  if(ToFile){
    Name1 <- paste("R", Run, "_Sites.csv", sep="")
    setwd(ResDir)
    write.table(Sites, Name1, sep=",", row.names=FALSE, col.names=TRUE)
    rm(Name1)
  }
  
  if(Run == RUNS[1]){
    SitesAllRuns <- Sites}
  
  if(Run != RUNS[1]){
    SitesAllRuns  <- rbind(SitesAllRuns, Sites)}
  
  rm(DataRF, PairsFull, Details, RF, AllRF, Sites, SumGDM, ModInfo)
} # End the Run loops

rm(Run)

################################################################################
### Write to file: SitesAllRuns, PairsAllRuns, SumGDMAllRuns, ModInfoAllRuns ###
################################################################################

# add info on the ground truth type and the treatment of the spectral/environmental varaibles
if(NMDS){
  Temp1 <- data.frame(TruthType   = rep(TruthName, nrow(SitesAllRuns)),
                      SpecEnviMod = rep("nMDS", nrow(SitesAllRuns)))
  Temp2 <- data.frame(TruthType   = rep(TruthName, nrow(PairsAllRuns)),
                      SpecEnviMod = rep("nMDS", nrow(PairsAllRuns)))
  Temp3 <- data.frame(TruthType   = rep(TruthName, nrow(SumGDMAllRuns)),
                      SpecEnviMod = rep("nMDS", nrow(SumGDMAllRuns)))
  Temp4 <- data.frame(TruthType   = rep(TruthName, nrow(ModInfoAllRuns)),
                      SpecEnviMod = rep("nMDS", nrow(ModInfoAllRuns)))
  
  SitesAllRuns   <- cbind(Temp1, SitesAllRuns)
  PairsAllRuns   <- cbind(Temp2, PairsAllRuns)
  SumGDMAllRuns  <- cbind(Temp3, SumGDMAllRuns)
  ModInfoAllRuns <- cbind(Temp4, ModInfoAllRuns)
  rm(Temp1, Temp2, Temp3, Temp4)
}
if(!NMDS){
  Temp1 <- data.frame(TruthType   = rep(TruthName, nrow(SitesAllRuns)),
                      SpecEnviMod = rep("AllVars", nrow(SitesAllRuns)))
  Temp2 <- data.frame(TruthType   = rep(TruthName, nrow(PairsAllRuns)),
                      SpecEnviMod = rep("AllVars", nrow(PairsAllRuns)))
  Temp3 <- data.frame(TruthType   = rep(TruthName, nrow(SumGDMAllRuns)),
                      SpecEnviMod = rep("AllVars", nrow(SumGDMAllRuns)))
  Temp4 <- data.frame(TruthType   = rep(TruthName, nrow(ModInfoAllRuns)),
                      SpecEnviMod = rep("AllVars", nrow(ModInfoAllRuns)))
  
  SitesAllRuns   <- cbind(Temp1, SitesAllRuns)
  PairsAllRuns   <- cbind(Temp2, PairsAllRuns)
  SumGDMAllRuns  <- cbind(Temp3, SumGDMAllRuns)
  ModInfoAllRuns <- cbind(Temp4, ModInfoAllRuns)
  rm(Temp1, Temp2, Temp3, Temp4)
}



if(ToFile){
  setwd(ResDir)
  write.table(SitesAllRuns,  "SitesAllRuns.csv",
              sep=",", row.names=FALSE, col.names=TRUE)
  write.table(PairsAllRuns,  "PairsAllRuns.csv",
              sep=",", row.names=FALSE, col.names=TRUE)
  write.table(SumGDMAllRuns, "SumGDMAllRuns.csv",
              sep=",", row.names=FALSE, col.names=TRUE)
  write.table(ModInfoAllRuns, "ModInfoAllRuns.csv",
              sep=",", row.names=FALSE, col.names=TRUE)
} # end Tofile if


# clear controls from global envio;
rm(DoPrep,         GDMVars,  InpDir,      Ntree,     
   ObjectID,  ObsKappaCol, ResDir,   RFVars,      RUNS,     
   ToFile,    Truth,       verbose,  AllFullGDM,  Join_Levels, 
   RunAllRF,  SingleGDM,   SitesGDM, WeiPropFull, WeiPropVotes, 
   SiteCol,   SpeDisRan,   EnvDisRan)


################################################################################
### End ---------------------------------------------------------------------###
################################################################################
