################################################################################
### Name:     MeanDis.R                                                      ###
### aim:                                                                     ###
### Calculate the mean change in value within a given distance range in each ###
### site.                                                                    ###
###                                                                          ###
### Input:                                                                   ###
### Data    - Dataframe with the required data.                              ###
### ObjCol  - Integer, the columnnumber with the objectIDs.                  ###
### SiteCol - Integer, the column number with the site IDs.                  ###
### ColX    - Integer, the column number with the X coordinate.              ###
### ColY    - Integer, the column number with the Y coordinate.              ###
### Var     - Vector of integers, the column number of variables to          ###
###           calcualte mean change in.                                      ###
### Range   - Vector of length 2 with the minimum and maximum distances to   ###
###           include in the mean. Minimum is not included, maximum is.      ###
### verbose - Logical, should progress be printed on consule.                ###
###                                                                          ###
### Process:                                                                 ###
### for each site:                                                           ###
### 1. Calcualte the spatial eucledian distance between all objects pairs    ###
###    using the dist() function.                                            ###
### 2. For each Var, calcualte the eucledian distance between all objects    ###
###    pairs using the dist() function.                                      ###
### 3. Create a logical matrix with TRUE when the range condition is met and ###
###    FALSE when it is not.                                                 ###
### 4. Use reshape:melt to change to 3 column mode, subset according to the  ###
###    range criteria and take the mean for each Var.                        ###
###                                                                          ###
### Output:                                                                  ###
### Dataframe with a row for each site (first column) and the mean eucledian ###
### distance for each varialbe in other columns. Variables names are:        ###
### Var_MinRange_MaxRange.                                                   ### 
###                                                                          ###
### Author: Yoni Gavish <gavishyoni@gmail.com>                               ###
### Date:   20 Sep. 2016                                                     ###
################################################################################

MeanDis = function(Data,
                   ObjCol,
                   SiteCol,
                   ColX,  
                   ColY,
                   Var,
                   Range,
                   verbose = T){
  
  
  ##############################################################################
  ### required packages                                                      ###
  ##############################################################################
  #require(randomForest)
  #require(caret)
  #require(StatMatch)     # the gower similarity
  #require(psych)         # descriptive statistics
  #require(vegan)          # for the bray-curtis dissimilarity
  require(reshape) # the melt function
  
  
  ##############################################################################
  ### Step 1 - create the output table                                       ###
  ##############################################################################
  if(verbose){
    cat(paste("##----- MeanDis --> Range:  ", 
              Range[1], 
              "-", 
              Range[2],
              "    Vars:  ",
              paste(names(Data)[Var], collapse=", "),
              "\n", 
              sep=""))
    
  }
  
  OutAll <- data.frame(Site = unique(Data[, SiteCol]))
  OutAll[, paste(names(Data)[Var], "_", Range[1],"_", Range[2], sep="" )] <- rep(NA, nrow(OutAll))
  
  
  ##############################################################################
  ### Step 2 - Loop over all sites                                           ###
  ##############################################################################
  if(verbose){cat(paste(  "## Site --> "))}
  
  for(i in 1:nrow(OutAll)){
    
    if(verbose){
      cat(paste(OutAll[i, 1], " --> ", sep=""))
    }
    
    ############################################################################
    ### Get the spatial eucledian distance                                   ###
    ############################################################################
    
    Data1              <- Data[Data[, SiteCol] == OutAll[i, 1], ]
    # get all Distances as a matrix
    DisFull <- as.matrix(dist(x = Data1[, c(ColX, ColY)], method = "euclidean"))
    colnames(DisFull) <- Data1[, ObjCol] 
    rownames(DisFull) <- Data1[, ObjCol]
    
    ############################################################################
    ### Get the distance for each Var                                        ###
    ############################################################################
    
    # vector of names for Var
    NAMES = paste(names(Data)[Var], "_", Range[1],"_", Range[2], sep="" )
    
    for (j in Var){

      DisTemp <- as.matrix(dist(x = Data1[, j], method = "euclidean"))
      colnames(DisTemp) <- Data1[, ObjCol] 
      rownames(DisTemp) <- Data1[, ObjCol]
      assign(paste(names(Data)[j], "_", Range[1],"_", Range[2], sep="" ), DisTemp)
      rm(DisTemp)
    }
    rm(j)
    
    ############################################################################
    ### check the distance range conditions                                  ###
    ############################################################################
   
    DisLog1 <- matrix(rep(FALSE,length(DisFull)),nrow= nrow(DisFull))
    DisLog2 <- DisLog1
    DisLog3 <- DisLog2
    DisLog1[DisFull >  Range[1]] <- TRUE
    DisLog2[DisFull <= Range[2]] <- TRUE
    DisLog3[DisLog1 & DisLog2]   <- TRUE
    colnames(DisLog3)            <- Data1[, ObjCol] 
    rownames(DisLog3)            <- Data1[, ObjCol]
    rm(DisLog1, DisLog2)
    
    ############################################################################
    ### Join all dsitances using reshape::melt                               ###
    ############################################################################
    
    library(reshape2)
    Temp1 <- setNames(melt(DisFull), c('Obj1', 'Obj2', 'Distance'))
    Temp2 <- setNames(melt(DisLog3), c('Obj1', 'Obj2', 'Range'))
    Temp1 <- cbind(Temp1, Temp2[,3])
    names(Temp1)[4] <- "Range"
    rm(Temp2, DisFull, DisLog3)
    
    for(k in 1:length(NAMES)){
      Data2 <- get(NAMES[k])
      Temp2 <- setNames(melt(Data2), c('Obj1', 'Obj2', 'Var'))
      Temp1 <- cbind(Temp1, Temp2[,3])
      names(Temp1)[4+k] <- NAMES[k]
      rm(Temp2, Data2)
    }
    rm(k)
    
    ############################################################################
    ### Subset according to Range, avcerage and insert to OutAll             ###
    ############################################################################
  
    Temp1 <- Temp1[Temp1$Range == TRUE, ]
    
    for(k in NAMES){
      OutAll[i, match(k, names(OutAll))] <- mean(Temp1[, k])
      
    }
    
    rm(list=NAMES)
    rm(k, Data1, Temp1, NAMES)
    
  } # end the OutAll loop
  if(verbose){
    cat(paste("Done\n"))
    }
  
  rm(i)
  return(OutAll)
  
} # end main function
    
  
    
    
    
   
