################################################################################
### Name:     RunAllRF.R                                                     ###
### aim:                                                                     ###
### Run all the RF analyses required for the distance decay of analysis      ###
###                                                                          ###
### Input:                                                                   ###
### DataRF   - data.frame, containing a unique object Id, the name of the    ###
###            site, the two ground-truth cateogires and all the spectral    ###
###            and enviornmental varialbes required to run the RF models.    ###
###            variables should include only those that can be assesed       ###
###            without any ground truth (i.e., that can be assessed for      ###
###            novel sites).                                                 ###
### Truth    - Integer, column number with the ground truth data.            ###
### Vars     - numeric vector, column numbers of RF explantory variables.    ###
### ObjectID - Integer, column number with the unique IDs of each object.    ###
### SiteCol  - Integer, coulmn number with the site IDs.                     ###
### Pairs    - dataframe with the pairs of site.                             ###
### ResDir   - Character, the directory in which to write the results.       ###
### Run      - Identifiers to add to all file names.                         ###
### NTree    - number of trees to grow in each F model.                      ###
### ToFile   - Logical, if T, Rf results wlll be written to file in ResDir.  ###        
### verbose  - Logical, should progress be printed on consule                ###
###                                                                          ###
### Procedure:                                                               ###
### 1. run all pairwise RF model, both local and global. create a file for   ###
###    each site in which the predicted local and global are stored. Also    ###
###    store the proportion of votes for each habitat according to the local ###
###    models. write this file to file.                                      ###
### 2. Estimate all the pairwise perfromance using the local models (kappa). ###
### 3. Estiamte the global model performance (Kappa) for each site.          ###
### 4. arrange the return list.                                              ###
###                                                                          ###
### Internal functions:                                                      ###
### Join_Levels.R                                                            ###
###                                                                          ###
### Output:                                                                  ###
### a list with three objects:                                               ###
### 1. AllRF   : list of dataframes, one for each site. Each data frame      ###
###              contains the objectID, SiteID, the groundtruth, the         ###
###              predcited class according to the local and global model and ###
###              the proportion of votes for each class according to the     ###
###              local model.                                                ###  
### 2. Pairs   : Dataframe, for each pair of sites, the kappa when training  ###
###              a model in the input site and predciting in the target      ###
###              site.                                                       ###
### 3. Sites   : Dataframe, containing  the site ID, the name of the         ###
###              file/object in AllRF that holds the RF output and the kappa ###
###              of the global model.                                        ###
###                                                                          ###
### Author: Yoni Gavish <gavishyoni@gmail.com>                               ###
### Date:   16 Sep. 2016                                                     ###
################################################################################



RunAllRF = function(DataRF, 
                    Truth, 
                    Vars, 
                    ObjectID, 
                    SiteCol, 
                    Pairs,
                    ResDir,
                    Run,
                    Ntree = 50,
                    ToFile  = T,
                    verbose = T){
  ##############################################################################
  #### required packages                                                     ###
  ##############################################################################
  require(randomForest)
  require(caret)
  
  ##############################################################################
  #### source functions                                                      ###
  ##############################################################################
  
  # The Join_Levels function
  # takes as input two vecotrs, turn them into factors, add to each one the 
  # levels that are not found in the other and sort the levels to be in the 
  # same order. Returns the two modified vectors.
  # order within each  vectors will be kept in the return list.
  Join_Levels = function(Vector_1,  # the first vector,  
                         Vector_2)  # the second vector
  { # Start function
    # change to factors
    Work_1 <- as.factor(Vector_1)
    Work_2 <- as.factor(Vector_2)
    
    # Extract the levels
    Work_1_Levels <- attributes(Work_1)$levels
    Work_2_Levels <- attributes(Work_2)$levels
    
    # the levels to add to each vector
    Work_1_Add <- Work_2_Levels[!(Work_2_Levels %in% Work_1_Levels)]
    Work_2_Add <- Work_1_Levels[!(Work_1_Levels %in% Work_2_Levels)]
    
    # add the levels to working vectors
    Work_1 <- factor(Work_1, levels=c(levels(Work_1), Work_1_Add))
    Work_2 <- factor(Work_2, levels=c(levels(Work_2), Work_2_Add))
    
    # sort the levels on both vectors
    Work_1 <- factor(Work_1, levels=sort(levels(Work_1)))
    Work_2 <- factor(Work_2, levels=sort(levels(Work_2)))
    
    # Check Work_1
    if(!all(as.character(Work_1)==as.character(Vector_1)))
    {  stop(paste("\n","Error in Join_Levels:", "\n",
                  "Unable to join the levels of the two vecotrs without changing the orignial data of Vector_1", "\n",
                  sep=""))  }
    
    # Check Work_2
    if(!all(as.character(Work_2)==as.character(Vector_2)))
    {  stop(paste("\n","Error in Join_Levels:", "\n",
                  "Unable to join the levels of the two vecotrs without changing the orignial data of Vector_2", "\n",
                  sep=""))  }
    
    # return the vectors
    Return_List <- list(Vector_1 = Work_1,
                        Vector_2 = Work_2)
    
  } # End function
  
  ##############################################################################
  #### Step 1 - run all pairwise RF                                          ###
  ##############################################################################
  
  
  if(verbose){
    cat(paste("\n##-----------------------------------------------------##\n"))
    cat(paste(  "## Step 1 - run all pairwise RF models:                ##\n"))
  }
  
  AllRF <- list()
  # all the sites to work on 
  SitesList <- unique(DataRF[, SiteCol])
  
  # the names of all the files that will be genrated in the loop
  FileList <- data.frame(Site = SitesList,
                         NAME  = rep(NA, length(SitesList)))
  
  for (i in SitesList){
    if(verbose){
      cat(paste("   Input site: ", i, " --> ",sep=""))
      #cat(paste("Extracting data --> "))
      }
    
    # the focal and test data
    LocalData  <- DataRF[DataRF[, SiteCol] == i, ]
    GlobalData <- DataRF[DataRF[, SiteCol] != i, ]
    
    
    if(verbose){cat(paste("Fitting RF (Local) --> "))}
    # run the randomForest model
    LocalRF <- randomForest(x     = LocalData[, Vars],
                            y     = droplevels(LocalData[, Truth]),
                            xtest = GlobalData[, Vars],                        
                            ntree = Ntree)
    if(verbose){cat(paste("Fitting RF (Global) --> "))}
    GlobalRF <- randomForest(x     = GlobalData[, Vars],
                             y     = droplevels(GlobalData[, Truth]),
                             xtest = LocalData[, Vars],                        
                             ntree = Ntree)
    
    #if(verbose){cat(paste("extract predicted classes --> "))}
    # insert back to local/global data
    LocalData$PredLocal   <- LocalRF$predicted
    LocalData$PredGlobal  <- GlobalRF$test[["predicted"]]
    GlobalData$PredLocal  <- LocalRF$test[["predicted"]]
    GlobalData$PredGlobal <- GlobalRF$predicted
    
    
    #if(verbose){cat(paste("extract votes --> "))}
    # votes of lcoal model in local data
    LocalVotes  <- as.data.frame(LocalRF$votes)
    # votes of local model in golbal data
    GlobalVotes <- as.data.frame(LocalRF$test[["votes"]])
    # all levels of habitats
    AllHab <- levels(DataRF[, Truth])
    
    for (k in AllHab){
      # the column in LocalVotes with the votes for habitat k
      # habitat k is not in the training data of the local model -LocCol will be NA
      # habitat k is in the training data of the local model
      LocCol <- match(k, names(LocalVotes))
      if(is.na(LocCol)){LocalData[, k]   <- rep(0, nrow(LocalData))}
      if(!is.na(LocCol)){LocalData[, k]  <- LocalVotes[, LocCol]}
      
      # the column in GlobalVotes with the votes for habitat k
      # habitat k is not in the training data of the local model -LocCol will be NA
      # habitat k is in the training data of the local model
      GlobCol <- match(k, names(GlobalVotes))
      if(is.na(GlobCol)){GlobalData[, k]  <- rep(0,nrow(GlobalData))}
      if(!is.na(GlobCol)){GlobalData[, k] <- GlobalVotes[, GlobCol]}
    }
      
    #if(verbose){cat(paste("rearrange data --> "))}
    FullData <- rbind(LocalData[, c(names(DataRF)[c(ObjectID, SiteCol, Truth)],"PredLocal", "PredGlobal", AllHab)],
                      GlobalData[, c(names(DataRF)[c(ObjectID, SiteCol, Truth)],"PredLocal", "PredGlobal", AllHab)])
    
    # reorder according to DataRF
    FullData <- FullData[match(DataRF[,ObjectID], FullData[,1]), ] 
    
    
    #if(verbose){cat(paste("store in list --> "))}
    # assign the FullData to an object and store objects name
    #AllRF[[as.character(paste("Run_",Run,"_Tr_", Truth, "_Site_" , i, "_RFOut", sep=""))]] <- FullData
    AllRF[[paste("R",Run, "_S" , i, "_RFOut", sep="")]] <- FullData
    
    
    
    #assign(x= paste("R",Run, "_S" , i, "_RFOut", sep=""), 
    #       value = FullData)
    FileList[i, 2] <- paste("R",Run, "_S" , i, "_RFOut", sep="")
    
    if(ToFile){
      # write results to file
      #if(verbose){cat(paste("write to file --> "))}
      setwd(ResDir)
      write.table(FullData, 
                  paste("R",Run,"_S" , i, "_RFOut.csv", sep=""), 
                  sep=",", 
                  row.names=FALSE, col.names=TRUE)
    }
    if(verbose){cat(paste("done\n"))}
    # remove stuff
    rm(LocalData, GlobalData, LocalRF, GlobalRF, AllHab, 
       LocalVotes, GlobalVotes, LocCol, GlobCol, FullData)
    
  }
  rm(i)  
  
  ##############################################################################
  #### Step 2 - Assess all pairwise performance                              ###
  ##############################################################################
  
  if(verbose){
    cat(paste("\n##-----------------------------------------------------##\n"))
    cat(paste("## Step 2 - Assess all pairwise performance (kappa):   ##\n "))
    cat(paste("##------ Pair --> "))
  }
  
  Pairs$ObsKappa  <-  NA
  
  for(i in 1: nrow(Pairs)){
    
    if(verbose){
      cat(paste( i ,", ", sep=""))
      #cat(paste("Retrieve RF data --> "))
      }
    
    InputID  <- Pairs[i, 2]
    TargetID <- Pairs[i, 3]
    
    # get the RFOut file
    NameRF <- FileList[match(InputID, FileList[,1]), 2]
    RFOut <- AllRF[[NameRF ]]
    
    # subset only the TargetID site
    TargetRF <- RFOut[RFOut[, 2] == TargetID, ]
    
    # the observed and predictor vectors
    Observed  <- TargetRF[, 3]
    Predicted <- TargetRF[, 4]
    
    #if(verbose){cat(paste("join factor levels --> "))}
    Temp <-  Join_Levels(Vector_1 = Observed,  # the first vector,  order will be kept in the return list
                         Vector_2 = Predicted)
    Observed  <- Temp[[1]]
    Predicted <- Temp[[2]]
    rm(Temp)
    
    #if(verbose){cat(paste(" kappa --> "))}
    Con_Mat <- confusionMatrix(data      = Predicted,
                               reference = Observed,
                               dnn = c("Predicted", "Observed")) 
    Perf <- Con_Mat$overall
    Pairs[i, "ObsKappa"] <-  Perf["Kappa"]
    #if(verbose){cat(paste("done\n"))}
    
    rm(InputID, TargetID, NameRF, RFOut, TargetRF, Observed, Predicted, Perf)
    
  }
  rm(i)
  if(verbose){cat(paste("done\n"))}
  
  ##############################################################################
  #### Step 3 - Assess the global Kappa for each site                        ###
  ##############################################################################
  
  if(verbose){
    cat(paste("\n##-----------------------------------------------------##\n"))
    cat(paste("## Step 3 - Assess the global performance (kappa):     ##\n"))
    cat(paste("##------ Site --> "))
  }
  
  FileList$GlobKappa <- NA
  FileList$SelfLocal <- NA
  FileList$MaxLocObs <- NA
  
  for (i in SitesList){
    
    if(verbose){
      cat(paste(i ,", ", sep=""))
      #cat(paste("Retrieve RF data --> "))
      }
    
    # get the RFOut file
    NameRF <- FileList[match(i, FileList[,1]), 2]
    RFOut <- AllRF[[NameRF ]]
    
    # subset only the focal site
    TargetRF <- RFOut[RFOut[, 2] == i, ]
    
    # the observed and predictor vectors
    Observed  <- TargetRF[, 3]
    Predicted <- TargetRF[, 5] # the global 
    
    #if(verbose){cat(paste("join factor levels --> "))}
    Temp <-  Join_Levels(Vector_1 = Observed,  # the first vector,  order will be kept in the return list
                         Vector_2 = Predicted)
    Observed  <- Temp[[1]]
    Predicted <- Temp[[2]]
    rm(Temp)
    
    #if(verbose){cat(paste(" kappa --> "))}
    Con_Mat <- confusionMatrix(data      = Predicted,
                               reference = Observed,
                               dnn = c("Predicted", "Observed")) 
    Perf <- Con_Mat$overall
    FileList[i, "GlobKappa"] <-  Perf["Kappa"]
    
    
    #### Add the SelfLocal kappa
    
    Temp1 <- Pairs[Pairs$InputID == i, ]
    FileList[i, "SelfLocal"] <- Temp1[Temp1$TargetID == i, "ObsKappa"]
    
    #### Add the MaxLocObs kappa
    Temp2 <- Pairs[Pairs$TargetID == i, ]
    Temp2 <- Temp2[Temp2$InputID != i, ]
    FileList[i, "MaxLocObs"] <- max(Temp2[, "ObsKappa"])
    
    #if(verbose){cat(paste("done\n"))}
    rm(NameRF, RFOut, TargetRF, Observed, Predicted, Perf, Temp1, Temp2)
     
  }
  if(verbose){cat(paste("done\n"))}
  
  ##############################################################################
  #### Step 4 - Arrange the return list                                      ###
  ##############################################################################
  
  #if(verbose){
  #  cat(paste(  "## Step 4 - Arraning the return list:                  ##\n"))}
  
  ReturnList <- list(AllRF = AllRF,
                     Pairs = Pairs,
                     Sites = FileList)
  #if(verbose){cat(paste("done\n"))}
  return(ReturnList)
 
  
}






