################################################################################
### Name:     SingleGDM.R                                                    ###
### aim:                                                                     ###
### Run a single GDM model, and extract all required values.                 ###
###                                                                          ###
### Input:                                                                   ###
### Data      - main data frame with all the data required for the gdm.      ###
### NewData   - Optional, if not NULL, the predcit function of gdm will be   ###
###             applied on the NewData, using Vars to indicate columns.      ###
### Depen     - the column number to use as the depndent variable.           ###
### Vars      - numeric vector with the column numbers to use as explanaotry ###
###             variables in the GDM analysis.                               ###
### Method    - Method by which the dependent variable is introduced into    ###
###             GDM:                                                         ###
###             "AsIS"       - the raw values are entered.                   ###
###             "Comp1"      - the complimentary to 1 is used.               ###
###             "Stan11"     - the range[-1, 1] is standardaised to the      ###
###                            range [0.1] as (value + 1)/2.                 ###
###             "CompStan11" - the range[-1, 1] is standardaised to the      ###
###                            range [0.1] as 1-((value + 1)/2).             ###
### geo, splines, knots - parameters of the gdm() function.                  ###
### Run       - the number of the Run (same as RunAllRf).                    ###
### ModName   - Character with the models name.                              ###
### verbose   - Logical, should progress be printed on consule.              ###
###                                                                          ###
### Process:                                                                 ###
### 1. Caculate the dependent distance based on 'Method' and 'Depen'.        ###
### 2. Run the GDM using the required 'Vars'.                                ###
### 3. Extract,deviance, coeficients and knots.                              ###
### 4. Extract Islines.                                                      ###
### 5. Extract predicted and ecological distance + backtrasform to kappa.    ###
###                                                                          ###
### Output:                                                                  ###
### A list with 3 objects:                                                   ###
### Data     - Same as input 'Data' with the 3 dditional columns for the     ###
###            predcited distance, predicted acological distance and         ###
###            predcited kappa. Columnnmae include 'ModName'.                ###
### SumGDM   - Info on the GDM models + goodness of fit measures +           ###
###            coefficients + knots.                                         ###
### ISplines - The X and Y required to plot all Isplines plots.              ###
### OutGDM   - The output of the gdm model.                                  ###
### NewData  - same as Data, only for NewData.                               ###
###                                                                          ###
### Author: Yoni Gavish <gavishyoni@gmail.com>                               ###
### Date:   23 Sep. 2016                                                     ###
################################################################################

# 
# Data    = PairsFull
# Depen   = 96
# Vars    = c(4:9, Details[, 4], Details[, 5])
# Method  = "AsIs"
# geo     = F 
# splines = NULL
# knots   = NULL
# Run     = 1
# ModName = "AllSites_All"
# verbose = T


SingleGDM = function(Data,
                     NewData = NULL,
                     Depen,
                     Vars,
                     Method  = "CompStan11",
                     geo     = F,
                     splines = NULL,
                     knots   = NULL,
                     Run, 
                     ModName,
                     verbose = T){
   
  
  ##############################################################################
  ### required packages                                                      ###
  ##############################################################################
  #require(randomForest)  # the random Forest models
  #require(caret)         # confusion matrix + kappa
  #require(StatMatch)     # the gower similarity
  #require(psych)         # descriptive statistics
  #require(vegan)         # for the bray-curtis dissimilarity
  require(gdm)            # all gdm analyses
  
  
  ##############################################################################
  ### Step 1 - Prepare the depndent variable                                 ###
  ##############################################################################
  
  if(verbose){
     cat(paste("##------- SingleGDM: ", ModName, " --> ",sep=""))
  }
  
  
  if(Method == "AsIs"){
    Data[, "distance"] <- Data[, Depen]} 
  
  if(Method == "Comp1"){
    Data[, "distance"] <- 1 - Data[, Depen]} 
  
  if(Method == "Stan11"){
    Data[, "distance"] <- (Data[, Depen] + 1) / 2} 
  
  if(Method == "CompStan11"){
    Data[, "distance"] <-  1- (Data[, Depen] + 1) / 2} 
  
  DataGDM <- Data[, Vars]
  
  class(DataGDM) <- c("gdmData", "data.frame")
  
  
  ##############################################################################
  ### Step 2 - Run the GDM                                                   ###
  ##############################################################################
  if(verbose){cat(paste("GDM --> "))}
  
  #### Run the gdm model
  OutGDM <- gdm(data = DataGDM, geo = geo, splines = splines, knots = knots)
  
  
  
  ##############################################################################
  ### Step 3 - Extract explained deviance, coeeficients and knots values     ###
  ##############################################################################
  
  if(verbose){cat(paste("Explained --> "))}
  #store in SumGDM
  SumGDM <- c(OutGDM$coefficients, OutGDM$knots)
  SumGDM <- as.data.frame(t(SumGDM))
  
  SumGDM2 <- data.frame()
  SumGDM2[1, 1] <- Run
  SumGDM2[1, 2] <- ModName
  SumGDM2[1, 3] <- OutGDM$gdmdeviance
  SumGDM2[1, 4] <- OutGDM$nulldeviance
  SumGDM2[1, 5] <- OutGDM$explained 
  SumGDM <- cbind(SumGDM2, SumGDM)
  rm(SumGDM2)
  
  # Add the columns names
  Pred       <- OutGDM$predictors
  SplinesOut <- OutGDM$splines
  NAMES1 <- c("Run", "ModName", "GDM_Deviance", "Null_Deviance", "Explained")
  NAMES2 <- c()
  NAMES3 <- c()
  
  for(k in 1:length(Pred)){
    PredName <- Pred[k]
    PredSpline <- SplinesOut[k]
    for(j in 1:PredSpline){
      coefName <- paste("Coef", j, "_", PredName, sep="")
      KnotName <- paste("Knot", j, "_", PredName, sep="")
      NAMES2 <- c(NAMES2, coefName)
      NAMES3 <- c(NAMES3, KnotName)
      rm(coefName, KnotName)
    } # End j loop
    rm(j, PredName, PredSpline)
  }# End Pred loop
  names(SumGDM)[] <- c(NAMES1, NAMES2, NAMES3)
  rm(k, Pred, SplinesOut, NAMES1, NAMES2, NAMES3)
  
  
  ##############################################################################
  ### Step 4 - Extract all Isplines values for future plotting               ###                                                ###
  ##############################################################################
  
  if(verbose){cat(paste("Isplines --> "))}
  # extract the x and y of the isplines
  Isplies_out   <- isplineExtract(OutGDM)
  TempX <- as.data.frame(Isplies_out[["x"]])
  TempY <- as.data.frame(Isplies_out[["y"]])
  # rename
  names(TempX) <- paste(names(TempX), "_Ix", sep="")
  names(TempY) <- paste(names(TempY), "_Iy", sep="")
  # join
  ISplines <- cbind(TempX, TempY)
  rm(Isplies_out, TempX, TempY)
  
  ##############################################################################
  ### Step 5 - backtransform and store predicted kappa                       ###                                                ###
  ##############################################################################
  ##-## the predicted and ecological distances are linked by: predicted = 1-exp(-ecological)
  #all(OutGDM$predicted == (1-exp(-OutGDM$ecological)))
  #all(round(OutGDM$ecological, 10) == round(-log(1-OutGDM$predicted),10))
  
  if(verbose){cat(paste("Kappa --> "))}
  # the predcited and ecological distances from GDM
  
  Data[, paste(ModName, "_PredGDM", sep="")]  <- OutGDM$predicted 
  Data[, paste(ModName, "_EcolGDM", sep="")]  <- OutGDM$ecological 

   
  # Back transform to Kappa if needed                    
  if(Method == "AsIs"){
    Data[, paste(ModName, "_Kappa", sep="")] <- OutGDM$predicted
    
    # predict for NewData and backtransform
    if(!is.null(NewData)){
      Temp1 <- predict(object = OutGDM, data = NewData[, Vars])
      NewData[, paste(ModName, "_PredGDM", sep="")]  <- Temp1
      NewData[, paste(ModName, "_EcolGDM", sep="")]  <- -log(1 - Temp1) 
      NewData[, paste(ModName, "_Kappa", sep="")]    <- Temp1
      rm(Temp1)
    }
  } 
  
  if(Method == "Comp1"){
    Data[, paste(ModName, "_Kappa", sep="")] <- 1- OutGDM$predicted
    
    # predict for NewData and backtransform
    if(!is.null(NewData)){
      Temp1 <- predict(object = OutGDM, data = NewData[, Vars])
      NewData[, paste(ModName, "_PredGDM", sep="")]  <- Temp1
      NewData[, paste(ModName, "_EcolGDM", sep="")]  <- -log(1 - Temp1) 
      NewData[, paste(ModName, "_Kappa", sep="")]    <- 1 -Temp1
      rm(Temp1)
    }
  } 
  
  if(Method == "Stan11"){
    Data[, paste(ModName, "_Kappa", sep="")] <- (2 * OutGDM$predicted) - 1
    
    # predict for NewData and backtransform
    if(!is.null(NewData)){
      Temp1 <- predict(object = OutGDM, data = NewData[, Vars])
      NewData[, paste(ModName, "_PredGDM", sep="")]  <- Temp1
      NewData[, paste(ModName, "_EcolGDM", sep="")]  <- -log(1 - Temp1) 
      NewData[, paste(ModName, "_Kappa", sep="")]    <- (2 * Temp1) - 1
      rm(Temp1)
    }
  } 
  
  if(Method == "CompStan11"){
    Data[, paste(ModName, "_Kappa", sep="")] <-  1 - (2 * OutGDM$predicted)
    
    # predict for NewData and backtransform
    if(!is.null(NewData)){
      Temp1 <- predict(object = OutGDM, data = NewData[, Vars])
      NewData[, paste(ModName, "_PredGDM", sep="")]  <- Temp1
      NewData[, paste(ModName, "_EcolGDM", sep="")]  <- -log(1 - Temp1) 
      NewData[, paste(ModName, "_Kappa", sep="")]    <- 1 - (2 * Temp1)
      rm(Temp1)
    }
  } 
  
  
  ##############################################################################
  ### Step 6 - arrange the reuturn list                                      ###
  ##############################################################################
  
  ReturnList <- list(Data     = Data,
                     SumGDM   = SumGDM,
                     ISplines = ISplines,
                     OutGDM   = OutGDM)
  
  if(!is.null(NewData)){
    ReturnList[["NewData"]] <- NewData 
  }
  
  
  if(verbose){cat(paste("done\n"))}
  return(ReturnList)
} # End main function


