################################################################################
### Name:     SitesGDM.R                                                     ###
### aim:                                                                     ###
### Run all the GDM models with each site set aside once.                    ###
###                                                                          ###
### Input:                                                                   ###
### Data      - main data frame with all the data required for the gdm.      ###
### Depen     - the column number to use as the depndent variable.           ###
### Vars      - numeric vector with the column numbers with the basic 6      ###
###             variables of GDM:  'distance', 'weights', 's1.xCoord',       ###         
###             's1.yCoord', 's2.xCoord', and 's2.yCoord'. THis is followed  ###
###             by all other variables in an                                 ###
###             's1.Var1' , 's1.Var2', ... 's2.Var1', 's2.Var2' format.      ###
### Method    - Method by which the dependent variable is introduced into    ###
###             GDM:                                                         ###
###             "AsIS"       - the raw values are entered.                   ###
###             "Comp1"      - the complimentary to 1 is used.               ###
###             "Stan11"     - the range[-1, 1] is standardaised to the      ###
###                            range [0.1] as (value + 1)/2.                 ###
###             "CompStan11" - the range[-1, 1] is standardaised to the      ###
###                            range [0.1] as 1-((value + 1)/2).             ###
### geo, splines, knots - parameters of the gdm() function.                  ###
### Run       - the number of the Run (same as RunAllRf).                    ###
### FunDir    - Character, the directory with the source functions.          ###
### ResDir    - Character, the directory in which to write the results.      ###
### ToFile    - Logical, if T, some results are written to file.             ###
### verbose   - Logical, should progress be printed on consule.              ###
###                                                                          ###
### Process:                                                                 ###
### 1. For each site, runs the SingleGDM function while setting the sites's  ###
###    kappa as both input and target aside.                                 ###
### 2. Use the GDM to predcit for the focal site, both as input and target.  ###
### 3. Collect in 'Kappa_Input' the predicted kappa that a model trained in  ###
###    the input site will have in any of the target sites when the input    ###
###    site is set aside.                                                    ###
### 4. collect in 'Kappa_Target' the predicted kappa that a model trained in ###
###    any of the input site will have on the Target site  when the Target   ###
###    site is set-aside.                                                    ###
### 5.  Store  all the predicted kappa of the GDMs in a specific column.     ###
### 6. For each SingleGDM:                                                   ###
###    2.a. - Collect the predcited, ecological distances + backtransoform   ###
###           to kappa.                                                      ###
###    2.b. - Write the ISplines to file.                                    ###
###    2.c. - Collect all te SumGDM values.                                  ###
###                                                                          ###
### Output:                                                                  ###
### A list with 2 objects:                                                   ###
### Data     - Same as input 'Data' with the additional columns for the      ###
###            predcited distance, predicted acological distance and         ###
###            predcited kappa for each of the GDM models. Column names      ###
###            include info on variable/sites used.                          ###
### SumGDM   - Info on the GDM models + goodness of fit measures +           ###
###            coefficients + knots for each of the 15 GDM models.           ###
###                                                                          ###
### Author: Yoni Gavish <gavishyoni@gmail.com>                               ###
### Date:   23 Sep. 2016                                                     ###
################################################################################




SitesGDM = function(Data,
                    #Details,
                    Depen,
                    Vars,
                    Method  = "CompStan11",
                    geo     = F,
                    splines = NULL,
                    knots   = NULL,
                    Run, 
                    FunDir,
                    ResDir,
                    ToFile,
                    verbose = T){
  
  
  
  ##############################################################################
  #### required packages                                                     ###
  ##############################################################################
  require(gdm)
  
  ##############################################################################
  #### source functions                                                      ###
  ##############################################################################
  # The Join_Levels function
  
  setwd(FunDir)
  source("SingleGDM.R")
  
  if(verbose){
    #cat(paste("\n##-----------------------------------------------------##"))
    cat(paste("\n##--- SitesGDM: --> \n"))
  }
  
  ##############################################################################
  #### Step1 - loop over all sites                                           ###
  ##############################################################################
  
  SitesList <- unique(Data[, 2])
  
  
  
  for (i in 1:length(SitesList)){
    
    # create Data and NewData
    Site <- SitesList[i]
    Data2   <- Data[Data$InputID != Site & Data$TargetID != Site, ]
    NewData <- Data[Data$TargetID == Site | Data$InputID == Site, ]
    ModName <- paste("Exclude",Site, "_All", sep="")
    
    Temp1 <- SingleGDM(Data    = Data2,
                       NewData = NewData, 
                       Depen   = Depen,
                       Vars    = Vars,
                       Method  = Method,
                       geo     = geo,
                       splines = splines,
                       knots   = knots,
                       Run     = Run, 
                       ModName = ModName,
                       verbose = verbose)
    # write Isplines to file
    if(ToFile){
      Temp2 <- Temp1[["ISplines"]]
      Name  <- paste("R", Run, "_",ModName, "_Splines.csv", sep="")
      setwd(ResDir)
      write.table(Temp2, Name, sep=",", row.names=FALSE, col.names=TRUE)
      rm(Temp2, Name)
    } # end ToFile if
    
    
    # Extract results
    Temp2 <- Temp1[["Data"]]
    Temp3 <- Temp1[["NewData"]]
    
    # Seperate Temp3 to input and target 
    Temp4 <- Temp3[Temp3$InputID  == Site, ] # input ID
    Temp5 <- Temp3[Temp3$TargetID == Site, ] # input ID
    
    # inset to Data
    # Kappa_Input - the expected kappa in the target site of a model fitted in the input site
    # Kappa_Target - 
    Data[match(Temp4$Pair_ID, Data$Pair_ID), "Kappa_Input"] <-  Temp4[, paste(ModName, "_Kappa", sep="")]
    Data[match(Temp5$Pair_ID, Data$Pair_ID), "Kappa_Target"] <- Temp5[, paste(ModName, "_Kappa", sep="")]
    
    
    # update Data for both Data and NewData in a single variable
    Data[match(Temp2$Pair_ID, Data$Pair_ID), paste(ModName, "_Kappa", sep="")] <- Temp2[, paste(ModName, "_Kappa", sep="")]
    Data[match(Temp3$Pair_ID, Data$Pair_ID), paste(ModName, "_Kappa", sep="")] <- Temp3[, paste(ModName, "_Kappa", sep="")]
    
    
    # Update the SumGDM
    if(i==1){
      SumGDM <- Temp1[["SumGDM"]]
      ROW <- 2
    }
    if(i!=1){
      SumGDM[ROW, ] <- Temp1[["SumGDM"]]
      ROW <- ROW +1
    }
    
    rm(Site, Data2, NewData, ModName, Temp1, Temp2, Temp3, Temp4, Temp5)
    
  } # End SitesList loop
  rm(i)
  
  ReturnList <- list(Data   = Data,
                     SumGDM = SumGDM)
  
  return(ReturnList)
} # End main function






