################################################################################
### Name:     StratGower.R                                                   ###
### aim:                                                                     ###
### Estimate the pairwise Gower distances between al pairs of sites,         ###
### stratified according to a categogorical variable.                        ###
###                                                                          ###
### Input:                                                                   ###
### DataGow  - data.frame, containing all variables at the object level.     ###
###            Must contain a column for site and a categorical varaibles on ###
###            which to base the stratification).                            ###
### Vars     - vector, column number with the variables to use for the Gower ###
###            distances.                                                    ###
### SiteCol  - Integer, coulmn number with the site IDs.                     ###
### Strat    - Integer, column number by which the pairwise distances will   ###
###            be stratified.                                                ###
### DoStrat  - Logical, if T, stratified means are returned based on Strat.  ###
### TEXT     - Character, text to add to column names to distinguish between ###
###            enironmental and spectral.                                    ### 
### Pairs    - dataframe with the pairs of site.                             ###
### verbose  - Logical, should progress be printed on consule.               ###
###                                                                          ###
### Process:                                                                 ###
### For each pair of sites:                                                  ###
### 1. Calcualte the mean and SD of the Gower distance between all object    ###
###    pairs.                                                                ###
### 2. if DoStrat - calcualte the mean (and SD) of the Gower distance        ###
###    between the two sites, seperately for each level of DataGow[, Strat]. ###
###                                                                          ###
### Output:                                                                  ###
### Data frame with a row for each pair of sites, with the mean and SD Gower ###
### distances based on all objects and stratified according to               ###
### DataGow[, Strat] level (if DoStrat = T). Names of column in output data  ###
### frame include TEXT for clarity in general workflow.                      ###
###                                                                          ###
### Author: Yoni Gavish <gavishyoni@gmail.com>                               ###
### Date:   19 Sep. 2016                                                     ###
################################################################################




StratGower = function(DataGow,
                      Vars,
                      SiteCol,
                      Strat,
                      DoStrat = F,
                      TEXT = "",
                      Pairs,
                      verbose = T){
  
  ##############################################################################
  ### required packages                                                      ###
  ##############################################################################
  #require(randomForest)
  #require(caret)
  require(StatMatch)     # the gower similarity
  require(psych)         # descriptive statistics
  
  ##############################################################################
  ### prepare the output dataframe                                           ###
  ##############################################################################
  
  if(verbose){
    cat(paste("##----- StratGower ------------------------------------##\n"))
  }
  
  # add variables for the mean and SD for the non-stratified results
  PairsOut <- Pairs
  PairsOut[, paste("All_Mean", TEXT, sep="")] <- NA
  PairsOut[, paste("All_SD", TEXT, sep="")]   <- NA
  
  if(DoStrat){
    # the levels of the categorical variable of the stratification
    HabLev <- levels(DataGow[, Strat])
    # add the mean and SD variables for each factor level of Strat
    for(k in HabLev){
      PairsOut[, paste(k ,"_Mean", TEXT, sep="")] <- NA
      PairsOut[, paste(k ,"_SD", TEXT, sep="")]   <- NA
    }
    rm(k)
  }
  ##############################################################################
  ###Loop over all pairs of sites                                            ###
  ##############################################################################
  
  for(i in 1:nrow(PairsOut)){
  
    InputID  <- PairsOut[i, 2]
    TargetID <- PairsOut[i, 3] 
    
    if(verbose){
      cat(paste("## Pair: ", 
                PairsOut[i, 1], 
                " (", 
                PairsOut[i, 2], 
                ",", 
                PairsOut[i, 3] , 
                ") --> ", 
                sep=""))
      
    }
    
    ############################################################################
    ### Input and target data + breaks                                       ###
    ############################################################################
    #if(verbose){cat(paste("Exrtracting Data --> "))}
    
    
    # Input and Target data for the pair
    InpData <- DataGow[DataGow[, SiteCol] == InputID, ]
    TarData <- DataGow[DataGow[, SiteCol] == TargetID, ]             
    
    # break the input  to 5 dataframes to ease computation
    Temp <- floor(nrow(InpData)/5)
    BREAKS <- c(0, Temp, Temp*2, Temp*3, Temp*4, nrow(InpData))
    rm(Temp)
    
    ############################################################################
    ### All_Mean + All_SD - non stratified Gower distances                   ###
    ############################################################################
    
    if(verbose){cat(paste("Gower (All objects) --> "))}
    
    for(j in 1:5){ 
      # subset of Inp1 data to work on, all input of target is used
      Inp1 <- InpData[c((BREAKS[j]+1): BREAKS[j+1]), ]
      
      # Gower distances
      Temp1 <- gower.dist(data.x = Inp1[, Vars],    # rows in output matrix
                          data.y = TarData[, Vars], # columns in output matrix 
                          rngs=NULL, 
                          KR.corr=TRUE)
      Temp2 <-as.vector(Temp1)
      if(j==1){AllGow <- Temp2}
      if(j!=1){AllGow <- c(AllGow, Temp2)}
      rm(Inp1, Temp1, Temp2)
      
    }
    
    Descrip <- describe(AllGow)
    PairsOut[i, paste("All_Mean", TEXT, sep="")] <-  Descrip$mean 
    PairsOut[i, paste("All_SD", TEXT, sep="")]   <-  Descrip$sd
    rm(AllGow, Descrip, BREAKS)
    rm(j)
    
    ############################################################################
    ### Stratified Gower distances                                           ###
    ############################################################################
    if(DoStrat){
      if(verbose){cat(paste("Gower (", names(DataGow)[Strat] ," factor levels) --> ", sep=""))}
      for(k in HabLev){
        # if(verbose){cat(paste(k, " --> ", sep=""))}
        # Subset factor level
        Inp1 <- InpData[InpData[, Strat] == k, ] 
        Tar1 <- TarData[TarData[, Strat] == k, ]
        
        # check there is something to work with
        if(nrow(Inp1)!=0 && nrow(Tar1) != 0){
          
          if(nrow(Inp1) > 900){
            Temp   <- floor(nrow(Inp1)/5)
            BREAKS <- c(0, Temp, Temp*2, Temp*3, Temp*4, nrow(Inp1))
            rm(Temp)}
          
          if(nrow(Inp1) <= 900){
            BREAKS <- c(0, nrow(Inp1))
          }
          
          
          for(j in 1:(length(BREAKS)-1)){
            Inp2 <- Inp1[c((BREAKS[j]+1): BREAKS[j+1]), ]
            
            # Gower distances
            Temp1 <- gower.dist(data.x = Inp2[, Vars],    # rows in output matrix
                                data.y = Tar1[, Vars], # columns in output matrix 
                                rngs=NULL, 
                                KR.corr=TRUE)
            Temp2 <-as.vector(Temp1)
            if(j==1){AllGow <- Temp2}
            if(j!=1){AllGow <- c(AllGow, Temp2)}
            rm(Inp2, Temp1, Temp2)
            
          } # end BREAKS loop
          
          Descrip <- describe(AllGow)
          PairsOut[i,  paste(k ,"_Mean", TEXT, sep="")] <-  Descrip$mean 
          PairsOut[i,  paste(k ,"_SD", TEXT, sep="")]   <-  Descrip$sd
          rm(AllGow, Descrip, BREAKS)
          rm(j)  
        }# End habitat is in data if
        
        rm(Inp1, Tar1)
        
      } #end HabLev loop
      rm(k)
    } # end DoStrat if
    if(verbose){
      cat(paste("Done\n"))}
    rm(InpData, TarData)
    
  }# End Pairs loop 
  
  
  
  return(PairsOut)
} # End function




  

  


