################################################################################
### Name:     WeiPropFull.R                                                  ###
### aim:                                                                     ###
### Calculate the weigted mean proportion of votes for each habitat for all  ###
### objects + kappa- for all target sites.                                   ###
###                                                                          ###
### Input:                                                                   ###
### AllRF   - list of dataframes, one for each site. Each dataframe contains ###
###           the objectID, SiteID, the groundtruth, the predcited class     ###
###           according to the local and global model and the proportion of  ###
###           votes for each class according to the local model. Output of   ###
###           RunAllRF.R  .                                                  ###
### WeiKapFull - Data frame, with five columns:                              ###
###             PairID - the ID of the pair of sites.                        ###
###             InputID - the site ID in which the model was trained.        ###    
###             TargerID- the focal site, a single value for all rows.       ###
###             NAME - the name of the file in AllRF, traine din the InputID ###
###             Kappa -to be used as weights.                                ###
### FunDir  - Character, the directory that stores functions to source.      ###
### WeiType - character the type of input weights. Calcualtion is the same,  ###
###           but the name of the column in the output would be different.   ###
###           "MeanLocEqu" - when equal weights are given to all models      ###
###           "MeanLocPre" - weights are the prediceted kappa (from GDM)     ###
###           "MeanLocObs" - weights are the prediceted kappa (from GDM)     ###                     
### verbose - logical, should progress be printed.                           ###
###                                                                          ###
### Process:                                                                 ###
### 1. Runs WeiPropVotes.R on all sites. Save the weigted proprotion of      ###
###    votes and kappa.                                                      ###
###                                                                          ###
### Output:                                                                  ###
### A list with two objects:                                                 ###
### AllObj  - data frame with the objectID, siteID, truth, and predcited     ###
###           class, as well as the weighted proportion of votes for each    ###
###           habitat. For the predicted, column name is WeiType. THe info   ###
###           in the site column is the ID of the target site.               ###
### SiteAll - Data frame. the perforamnce (Kappa) of WeiType in each site.   ###
###           First column is site and second is the kappa.                  ###
###                                                                          ###
### Author: Yoni Gavish <gavishyoni@gmail.com>                               ###
### Date:   19 Sep. 2016                                                     ###
################################################################################


WeiPropFull = function(AllRF, 
                       WeiKapFull,
                       FunDir,
                       WeiType,
                       verbose){
  
  ##############################################################################
  #### source functions                                                      ###
  ##############################################################################
  
  # The WeiPropVotes function
  setwd(FunDir)
  source("WeiPropVotes.R")
  
  ##############################################################################
  #### required packages                                                     ###
  ##############################################################################
  #require(caret)
  
  ##############################################################################
  #### Step 1 - Standardize weightsd to 1                                    ###
  ##############################################################################
  
  # list of sites:
  SiteAll <- as.data.frame(unique(WeiKapFull$TargetID))
  names(SiteAll)[1] <- "Site"
  SiteAll[, WeiType] <- NA
  
  if(verbose){
    #cat(paste("##-----------------------------------------------------##\n"))
    cat(paste(  "\n##--- WeiPropFull:      ", WeiType, "\n", sep=""))
  }
  
  for (i in 1:nrow(SiteAll)){
    
    # subset WEiKapFull + remove the self local
    WeiKap <- WeiKapFull[WeiKapFull$TargetID == SiteAll[i, 1], ]
    WeiKap <- WeiKap[WeiKap$InputID != WeiKap$TargetID, ]
    
    # Call WeiPropVotes
    Out1 <- WeiPropVotes(AllRF     = AllRF, 
                         WeiKap    = WeiKap,
                         WeiType   = WeiType,
                         verbose   = verbose)
   
    
    
    if(i==1){AllObj <- Out1[["AllObj"]]}
    if(i!=1){AllObj <- rbind(AllObj, Out1[["AllObj"]])}
    
    
    SiteAll[i, WeiType] <- as.numeric(Out1[["KAPPA"]])
    rm(WeiKap, Out1)
    
  }
  
  return(list(AllObj  = AllObj,
              SiteAll = SiteAll))
  
  
} # End main function
  
  
  
  