################################################################################
### Name:     WeiPropVotes.R                                                 ###
### aim:                                                                     ###
### Calculate the weigted mean proportion of votes for each habitat for all  ###
### objects + kappa.                                                         ###
###                                                                          ###
### Input:                                                                   ###
### AllRF   - list of dataframes, one for each site. Each dataframe contains ###
###           the objectID, SiteID, the groundtruth, the predcited class     ###
###           according to the local and global model and the proportion of  ###
###           votes for each class according to the local model. Output of   ###
###           RunAllRF.R  .                                                  ###
### WeiKap  - Data frame, with five columns:                                 ###
###             PairID - the ID of the pair of sites.                        ###
###             InputID - the site ID in which the model was trained.        ###    
###             TargerID- the focal site, a single value for all rows.       ###
###             NAME - the name of the file in AllRF, traine din the InputID ###
###             Kappa -to be used as weights.                                ###
### FunDir  - Character, the directory that stores functions to source.      ###
### WeiType - character the type of input weights. Calcualtion is the same,  ###
###           but the name of the column in the output would be different.   ###
###           "MeanLocEqu" - when equal weights are given to all models      ###
###           "MeanLocPre" - weights are the prediceted kappa (from GDM)     ###
###           "MeanLocObs" - weights are the prediceted kappa (from GDM)     ###                     
### verbose - logical, should progress be printed.                           ###
###                                                                          ###
### Process:                                                                 ###
### 1. Standardize the weights to sum to 1.                                  ###
### 2. Extract the proprtion of votes from allt he LocalRF models. Then      ###
###    calucalte the portpotion*weight value.                                ###
### 3. Calcualte the weighted mean for all objects.                          ###
### 4. Assess performance - Kappa.                                           ###
###                                                                          ###
### Output:                                                                  ###
### List with 2 objects:                                                     ###
### AllObj - data frame, with the objectId, SIte, Truth and predcited value, ###
###          followed by the weighted mean porportion for each class. The    ###
###          column name of the predicted value is 'WeiType'.                ###
### KAPPA  - Numeric, the value o fthe kappa statistic.                      ###
###                                                                          ###
### Author: Yoni Gavish <gavishyoni@gmail.com>                               ###
### Date:   19 Sep. 2016                                                     ###
################################################################################

## example input for WeiKap
# # output of RunAllRF
# AllRF  <- Out_RunAllRF[["AllRF"]]
# Pairs2 <- Out_RunAllRF[["Pairs"]]
# Sites  <- Out_RunAllRF[["Sites"]]
# # prepare an example of WeiKap
# WeiKap <- Pairs2[Pairs2$TargetID ==3,]
# # remove the self local
# WeiKap <- WeiKap[WeiKap$InputID != WeiKap$TargetID, ]
# # add the names of files from Sites
# WeiKap$NAME <- Sites[match(WeiKap$InputID, Sites[,1]), "NAME" ]
# # reorder:
# WeiKap <- WeiKap[, c(1,2,3,5,4)]


WeiPropVotes = function(AllRF, 
                        WeiKap,
                        WeiType,
                        verbose){
  
  
  ##############################################################################
  #### required packages                                                     ###
  ##############################################################################
  #require(randomForest)
  require(caret)
  
  ##############################################################################
  #### source functions                                                      ###
  ##############################################################################
  
  # The Join_Levels function
  # takes as input two vecors, turn them into factors, add to each one the 
  # levels that are not found in the other and sort the levels to be in the 
  # same order. Returns the two modified vectors.
  # order within each vectors will be kept in the return list.
  Join_Levels = function(Vector_1,  # the first vector,  
                         Vector_2)  # the second vector
  { # Start function
    # change to factors
    Work_1 <- as.factor(Vector_1)
    Work_2 <- as.factor(Vector_2)
    
    # Extract the levels
    Work_1_Levels <- attributes(Work_1)$levels
    Work_2_Levels <- attributes(Work_2)$levels
    
    # the levels to add to each vector
    Work_1_Add <- Work_2_Levels[!(Work_2_Levels %in% Work_1_Levels)]
    Work_2_Add <- Work_1_Levels[!(Work_1_Levels %in% Work_2_Levels)]
    
    # add the levels to working vectors
    Work_1 <- factor(Work_1, levels=c(levels(Work_1), Work_1_Add))
    Work_2 <- factor(Work_2, levels=c(levels(Work_2), Work_2_Add))
    
    # sort the levels on both vectors
    Work_1 <- factor(Work_1, levels=sort(levels(Work_1)))
    Work_2 <- factor(Work_2, levels=sort(levels(Work_2)))
    
    # Check Work_1
    if(!all(as.character(Work_1)==as.character(Vector_1)))
    {  stop(paste("\n","Error in Join_Levels:", "\n",
                  "Unable to join the levels of the two vecotrs without changing the orignial data of Vector_1", "\n",
                  sep=""))  }
    
    # Check Work_2
    if(!all(as.character(Work_2)==as.character(Vector_2)))
    {  stop(paste("\n","Error in Join_Levels:", "\n",
                  "Unable to join the levels of the two vecotrs without changing the orignial data of Vector_2", "\n",
                  sep=""))  }
    
    # return the vectors
    Return_List <- list(Vector_1 = Work_1,
                        Vector_2 = Work_2)
    
  } # End function
  
  ##############################################################################
  #### Step 1 - Standardize weights to 1                                     ###
  ##############################################################################
  
  TargetID <- unique(WeiKap[, 3])
  # check
  if(length(TargetID) != 1){
    stop(paste("\n WeiPropVotes: Error- more than 1 TargetID in WeiKap\n"))
  }
  
  if(verbose){
    cat(paste(  "##-------- WeiPropVotes:   Target Site -->     ", TargetID, "\n", sep=""))
    #cat(paste(  "## Standardize weights --> "))
  }
  WeiKap[, "Weights"] <- WeiKap[, 5] / sum(WeiKap[, 5])
  
  
  # prepare the output dataframe
  AllObj                      <- AllRF[[1]]
  AllObj                      <- AllObj[AllObj[, 2] == TargetID, ]
  AllLev                      <- names(AllObj)[6:ncol(AllObj)]
  AllObj                      <- AllObj[, c(1:3, 6:ncol(AllObj))] 
  AllObj[, c(4:ncol(AllObj))] <- NA 
  AllObj[, WeiType]           <- NA
  AllObj[, WeiType]           <- factor(AllObj[, WeiType], levels = AllLev)
  
  #if(verbose){cat(paste("done\n"))}
  
  ##############################################################################
  #### Step 2 - Collect the votes from all RF                                ###
  ##############################################################################
  
  if(verbose){
    #cat(paste("\n##-----------------------------------------------------##\n"))
    cat(paste(  "##------------ Collect votes --> "))
    #cat(paste(  "## Collect all votes for Target site  -->   Input site: "))
  }
  
  for (i in 1:nrow(WeiKap)){
    #if(verbose){
    #  cat(paste(WeiKap[i, 2], " --> ",sep=""))}
  
    # get the randomForest data
    RFOut <- AllRF[[WeiKap[i,"NAME"]]]
    # subset only targetId objects
    RFOut <- RFOut[RFOut$Site == TargetID, ]
    # add the input site and weight
    RFOut[, "InputSite"] <- rep(WeiKap[i, 2], nrow(RFOut))
    RFOut[, "Weight"]    <- rep(WeiKap[i, 6], nrow(RFOut))
    # add all prop multiplied by the weight
    RFOut[, paste("Wei_", AllLev, sep="")] <- RFOut[, AllLev] * RFOut[, "Weight"]
    if(i==1){
      AllProp <- RFOut
    }
    if(i!=1){
      AllProp <- rbind(AllProp, RFOut)
    }
    rm(RFOut)
  }
  
  #if(verbose){cat(paste("done\n"))}
  rm(i)
  
  ##############################################################################
  #### Step 3 - Calculate the weighted mean proportion of votes              ###
  ##############################################################################
  
  if(verbose){
    #cat(paste("\n##-----------------------------------------------------##\n"))
    cat(paste(  "Weighted mean  --> "))
    #cat(paste(  "## Calculate weighted mean proportions for object: --> "))
  }
  
  for(i in 1:nrow(AllObj)){
    #if(verbose & round(i/200,0) == i/200){cat(paste(i, " --> ", sep=""))}
    Prop <- AllProp[AllProp[, 1] == AllObj[i, 1],]
    #AllObj[i, "Site"] <- unique(Prop[, 2])
    #AllObj[i, "Truth"] <- unique(Prop[, 3])
   
    Temp1 <- colSums(Prop[, paste("Wei_", AllLev, sep="")])
    
    # check all the names of habitats match
    if(all(paste("Wei_", names(AllObj)[4:(ncol(AllObj)-1)], sep="") == names(Temp1))){
      AllObj[i, 4:(ncol(AllObj)-1)] <- Temp1
    }
    if(!all(paste("Wei_", names(AllObj)[4:(ncol(AllObj)-1)], sep="") == names(Temp1))){
      stop(paste("WeiPropVotes: can't match all habitats for object: ", AllObj[i, 1], sep=""))
    }
    
    AllObj[i, WeiType] <- names(AllObj)[match(max(Temp1), AllObj[i, ])]
    
    
    rm(Prop, Temp1)
  }
  
  #if(verbose){cat(paste("done\n"))}
  
  ##############################################################################
  #### Step 4 - asessing performance                                         ###
  ##############################################################################
  
  if(verbose){
    #cat(paste("\n##-----------------------------------------------------##\n"))
    cat(paste(  "kappa --> "))
  }
  
  # the observed and predictor vectors
  Observed  <- AllObj[, 3]
  Predicted <- AllObj[, WeiType]
  
  Temp <-  Join_Levels(Vector_1 = Observed,  # the first vector,  order will be kept in the return list
                       Vector_2 = Predicted)
  Observed  <- Temp[[1]]
  Predicted <- Temp[[2]]
  rm(Temp)
  
  Perf <- confusionMatrix(data      = Predicted,
                          reference = Observed,
                          dnn = c("Predicted", "Observed"))$overall["Kappa"]
  
  if(verbose){
    cat(paste("done\n"))
    #cat(paste("##-----------------------------------------------------##\n"))
    }
  
  ##############################################################################
  #### Step 5 - return                                                       ###
  ##############################################################################
  #reorder AllObj
  AllObj <- AllObj[, c(1:3, ncol(AllObj), 4:(ncol(AllObj)-1))]
  
  return(list(AllObj =AllObj,
              KAPPA = Perf))
  
}